package com.android.launcher3.anim

import android.view.View
import androidx.annotation.Keep

open class NullableViewSet(open vararg val views: View?) {
    @Keep
    fun setAlpha(alpha: Float) {
        views.forEach { it?.alpha = alpha }
    }

    fun getAlpha(): Float = views.filterNotNull().firstOrNull()?.alpha ?: 1.0f

    @Keep
    fun setTranslationX(trans: Float) {
        views.forEach { it?.translationX = trans }
    }

    @Keep
    fun setTranslationY(trans: Float) {
        views.forEach { it?.translationY = trans }
    }

    @Keep
    fun setScaleX(scale: Float) {
        views.forEach { it?.scaleX = scale }
    }

    @Keep
    fun setScaleY(scale: Float) {
        views.forEach { it?.scaleY = scale }
    }

    @Keep
    fun setScale(scale: Float) {
        views.forEach {
            it?.scaleY = scale
            it?.scaleX = scale
        }
    }
}

class ViewSet(override vararg val views: View): NullableViewSet(*views) {

    init {
        views.forEachIndexed { index, view -> requireNotNull(view) { "ViewSet views[$index] is null and must not be"} }
    }

}