package com.teslacoilsw.kotlin

import android.graphics.Bitmap
import android.graphics.Rect
import android.graphics.drawable.BitmapDrawable
import android.graphics.drawable.Drawable
import androidx.core.graphics.applyCanvas
import androidx.core.graphics.createBitmap
import com.android.launcher3.FastBitmapDrawable
import com.teslacoilsw.launcher.icon.handleAdaptiveIcon


fun Drawable.asBitmap(desiredWidth: Int, desiredHeight: Int = desiredWidth): Bitmap {

    handleAdaptiveIcon(this, desiredWidth)?.let { return it }

    return when (this) {
        is BitmapDrawable -> return bitmap
        is FastBitmapDrawable -> return bitmap
        else -> {
            val bmp = createBitmap(desiredWidth, desiredHeight)
            bmp.applyCanvas {
                var w = intrinsicWidth
                var h = intrinsicHeight
                if (w <= 0 || h <= 0) {
                    w = desiredWidth
                    h = desiredHeight
                }
                val oldBounds = Rect(bounds)
                setBounds(0, 0, w, h)
                if (w != desiredWidth || h != desiredHeight)
                    scale(desiredWidth/w.toFloat(), desiredHeight/h.toFloat())
                draw(this)
                bounds = oldBounds
            }

        }
    }
}