package com.teslacoilsw.launcher

import android.app.Fragment
import android.content.Intent
import android.graphics.drawable.ColorDrawable
import android.os.Bundle
import android.text.Editable
import android.text.TextWatcher
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.EditText
import android.widget.TextView
import androidx.appcompat.app.AppCompatActivity
import androidx.core.view.forEach
import androidx.core.view.get
import androidx.core.view.isNotEmpty
import com.teslacoilsw.launcher.preferences.SettingsActivity
import com.teslacoilsw.launcher.preferences.fancyprefs.FancyPrefCardBorderView
import com.teslacoilsw.launcher.preferences.fancyprefs.FancyPrefView
import com.teslacoilsw.launcher.preferences.fragments.SettingsDesktop
import com.teslacoilsw.launcher.preferences.fragments.SettingsDock
import com.teslacoilsw.launcher.preferences.fragments.SettingsDrawer
import com.teslacoilsw.launcher.preferences.fragments.SettingsFolder
import com.teslacoilsw.launcher.preferences.fragments.SettingsGestures
import com.teslacoilsw.launcher.preferences.fragments.SettingsIntegrations
import com.teslacoilsw.launcher.preferences.fragments.SettingsLookFeel
import com.teslacoilsw.launcher.preferences.fragments.SettingsNightMode
import com.teslacoilsw.launcher.search.match

class NovaSettingsSearchActivity : AppCompatActivity() {
    private val resultView by lazy { findViewById<ViewGroup>(R.id.search_result_content) }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_nova_settings_search)
        window.setBackgroundDrawable(ColorDrawable(0x40000000.toInt()))

        loadFragment(SettingsDesktop())
        loadFragment(SettingsDock())
        loadFragment(SettingsDrawer())
        loadFragment(SettingsGestures())
        loadFragment(SettingsFolder())
        loadFragment(SettingsLookFeel())
        loadFragment(SettingsNightMode())
        loadFragment(SettingsIntegrations())

        findViewById<View>(R.id.roundRectView).clipToOutline = true

        findViewById<View>(R.id.searchbar_icon).setOnClickListener {
            finish()
        }
        findViewById<View>(R.id.sneeze_guard).setOnClickListener {
            finish()
        }

        val searchTextView = findViewById<EditText>(R.id.searchbar)
        searchTextView.addTextChangedListener(object: TextWatcher {
            override fun afterTextChanged(s: Editable) {
                val query = s.toString()
                val container = findViewById<ViewGroup>(R.id.fragment_container)
                val desktop = container[0] as ViewGroup
                val dock = container[1] as ViewGroup
                val drawer = container[2] as ViewGroup
                val gestures = container[3] as ViewGroup
                val folder = container[4] as ViewGroup
                val lookFeel = container[5] as ViewGroup
                val nightMode = container[6] as ViewGroup
                val integrations = container[7] as ViewGroup

                resultView.removeAllViews()

                val inflater = LayoutInflater.from(resultView.context)

                fun processCategory(targetId: String, title: String, layout: ViewGroup) {
                    val result = layout.searchViewTree(query)
                    if (!result.isEmpty()) {
                        if (resultView.isNotEmpty())
                            resultView.addView(FancyPrefCardBorderView(resultView.context))

                        val cat = inflater.inflate(R.layout.novasettings_search_sectionheader, resultView, false) as TextView
                        cat.text = title
                        resultView.addView(cat)
                        for (x in result) {
                            val row = FancyPrefView<Unit>(resultView.context)
                            row.id = x.id
                            row.title = x.title
                            if (x.summaryActualText?.isNotBlank() == true) {
                                row.summary = x.summaryActualText
                            }

                            row.setOnClickListener {
                                val intent = Intent(this@NovaSettingsSearchActivity, SettingsActivity::class.java)
                                intent.putExtra("target", targetId)
                                intent.putExtra("targetPref", row.id)
                                intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP)
                                startActivity(intent)
                                finish()
                            }
                            resultView.addView(row)
                        }
                    }
                }
                fun processCategory(targetId: String, title: Int, layout: ViewGroup) {
                    processCategory(targetId, getString(title), layout)
                }

                if (s.isNotBlank()) {
                    processCategory("desktop",R.string.preference_header_desktop, desktop)
                    processCategory("dock", getString(R.string.preference_header_desktop) + " > " + getString(R.string.preference_header_dock), dock)
                    processCategory("drawer", R.string.preference_header_drawer, drawer)
                    processCategory("gestures", R.string.preference_header_gestures_inputs, gestures)
                    processCategory("folder", R.string.preference_header_folders, folder)
                    processCategory("lookfeel", R.string.preference_header_look_feel, lookFeel)
                    processCategory("nightmode", R.string.pref_night_mode, nightMode)
                    processCategory("integrations", R.string.preference_header_integrations, integrations)
                }
            }

            override fun beforeTextChanged(s: CharSequence, start: Int, count: Int, after: Int) {
            }

            override fun onTextChanged(s: CharSequence, start: Int, before: Int, count: Int) {
            }

        })

        findViewById<View>(R.id.searchbar_icon_clear).setOnClickListener {
            searchTextView.setText("")
        }

    }

    fun ViewGroup.searchViewTree(s: String): List<FancyPrefView<*>> {
        val result = mutableListOf<FancyPrefView<*>>()
        forEach {
            if (it is FancyPrefView<*> && it.id != 0 && it.includeInSearch()) {
                if (match(it.title ?: "", s) > 0|| match(it.summaryActualText ?: "", s) > 0 || match(it.searchKeywords ?: "", s) > 0) {
                    result.add(it)
                }
            } else if (it is ViewGroup) {
                result.addAll(it.searchViewTree(s))
            }
        }
        return result
    }


    private fun loadFragment(f: Fragment) {
        this.fragmentManager.beginTransaction()
                .add(R.id.fragment_container, f, "SETTINGS_FRAGMENT")
                .commit()
    }


}
