package com.teslacoilsw.launcher.backup

import android.content.Context
import android.graphics.Bitmap
import android.net.Uri
import android.os.Environment
import androidx.core.net.toFile
import com.android.launcher3.LauncherAppState
import com.android.launcher3.LauncherProvider
import com.teslacoilsw.kotlin.tryCatch
import com.teslacoilsw.launcher.NovaLauncher
import com.teslacoilsw.launcher.onboarding.BackupPreviewGenerator
import java.io.File

fun addAllApexBackups(backupList: MutableList<BackupRestoreAdapter.BackupAdapterItem>): Boolean {
    val path = File(Environment.getExternalStorageDirectory(), "Android/data/apexlauncher")
    var added = false
    val files = path.listFiles()
    if (files != null) {
        files.sortByDescending { it.lastModified() }
        for (f in files) {
            if (f.isFile) {
                if (f.absolutePath.endsWith("apexbackup")) {
                    backupList.add(BackupFileApex40(Uri.fromFile(f)))
                    added = true
                } else if (f.name == "apex_settings.bak") {
                    backupList.add(BackupFileApex33Local(Uri.fromFile(f)))
                    added = true
                }
            }
        }
    }
    return added
}

class BackupFileApex40(uri: Uri) : BackupRestoreAdapter.BackupUriFile(uri) {
    override fun doRestore(context: Context) {
        LauncherAppState.getInstance().model.stopLoader()
        synchronized(LauncherProvider.sWriteLock) {
            LauncherAppState.getLauncherProvider()?.closeHard()
            NovaLauncher.getLauncher()?.finish()

            ApexBackup.loadFromZip(context, uri)
        }
    }

    override fun getName(context: Context): String {
        return "Apex " + uri.lastPathSegment.removeSuffix(".apexbackup")
    }

    override fun getLastModifiedTime(): Long {
        if (uri.scheme == "file") {
            return tryCatch(0L) { uri.toFile().lastModified() }
        }
        return 0L
    }

    override fun generatePreview(context: Context, width: Int, height: Int): Bitmap? {
        val bmp = Bitmap.createBitmap(width, height, Bitmap.Config.ARGB_8888)
        try {
            val data = ApexBackup.loadImportedDataFromZip(context, uri)
            BackupPreviewGenerator.generatePreview(bmp, data)
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return bmp
    }
}

class BackupFileApex33Local(uri: Uri) : BackupRestoreAdapter.BackupUriFile(uri) {
    override fun getName(context: Context?): String {
        return "Apex 3.3 backup"
    }

    override fun doRestore(context: Context) {
        LauncherAppState.getInstance().model.stopLoader()
        synchronized(LauncherProvider.sWriteLock) {
            LauncherAppState.getLauncherProvider()?.closeHard()
            NovaLauncher.getLauncher()?.finish()

            ApexBackup.loadFromDirectory(context, uri.toFile().parentFile)
        }
    }

    override fun getLastModifiedTime(): Long {
        if (uri.scheme == "file") {
            return tryCatch(0L) { uri.toFile().lastModified() }
        }
        return 0L
    }
}