package com.teslacoilsw.launcher.deepshortcutproxy

import android.content.pm.LauncherApps
import android.content.pm.ShortcutInfo
import android.os.Build
import android.os.Build.VERSION.SDK_INT
import android.os.Build.VERSION_CODES.P
import android.os.ParcelFileDescriptor
import android.os.UserHandle
import androidx.annotation.RequiresApi
import com.teslacoilsw.shared.util.ReflectionHelper

private val _hasIconResource = if (SDK_INT < P) ReflectionHelper.findPrivateMethod(ShortcutInfo::class.java, "hasIconResource") else null
fun ShortcutInfo.reflectHasIconResource(): Boolean {
    _hasIconResource?.let {
        try {
            return it.invoke(this) as Boolean
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }
    return false
}

private val _hasIconFile = if (SDK_INT < P) ReflectionHelper.findPrivateMethod(ShortcutInfo::class.java, "hasIconFile") else null
fun ShortcutInfo.reflectHasIconFile(): Boolean {
    _hasIconFile?.let {
        try {
            return it.invoke(this) as Boolean
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }
    return false
}

private val _getShortcutIconFd = if (SDK_INT < P) ReflectionHelper.findPrivateMethod(LauncherApps::class.java,
        "getShortcutIconFd", String::class.java, String::class.java, UserHandle::class.java) else null
@RequiresApi(Build.VERSION_CODES.N_MR1)
fun LauncherApps.reflectionGetShortcutIconFd(shortcutInfo: ShortcutInfo): ParcelFileDescriptor? {
    _getShortcutIconFd?.let {
        try {
            return it.invoke(this, shortcutInfo.`package`, shortcutInfo.id, shortcutInfo.userHandle) as ParcelFileDescriptor
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }
    return null
}

private val _getIconResourceId = if (SDK_INT < P) ReflectionHelper.findPrivateMethod(ShortcutInfo::class.java, "getIconResourceId") else null
fun ShortcutInfo.reflectGetIconResourceId(): Int {
    _getIconResourceId?.let {
        try {
            return it.invoke(this) as Int
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }
    return 0
}