package com.teslacoilsw.launcher.folder

import android.graphics.Canvas
import android.graphics.ColorFilter
import android.graphics.Paint
import android.graphics.Path
import android.graphics.PixelFormat
import android.graphics.Rect
import android.graphics.drawable.Drawable
import androidx.annotation.ColorInt

class ImmersiveFolderDrawable(@get:ColorInt @param:ColorInt val innerColor: Int, @get:ColorInt @param:ColorInt val outerColor: Int, val radius: Float) : Drawable() {
    private val paint = Paint(Paint.ANTI_ALIAS_FLAG)
    private val path = Path()
    private val outerPath = Path()
    private val padding = Rect()

    var bottomCorners: Boolean = true
        set(x) { field = x; updateBounds(bounds)}

    fun setPadding(left: Int, top: Int, right: Int, bottom: Int) {
        padding.set(left, top, right, bottom)
        updateBounds(bounds)
        invalidateSelf()
    }

    private fun updateBounds(bounds: Rect) {
        path.reset()

        if (bottomCorners)
            path.addRoundRect(padding.left.toFloat(), padding.top.toFloat(), bounds.width().toFloat() - padding.right, bounds.height().toFloat() - padding.bottom, radius, radius, Path.Direction.CCW)
        else
            path.addRoundRect(padding.left.toFloat(), padding.top.toFloat(), bounds.width().toFloat() - padding.right, bounds.height().toFloat() - padding.bottom, floatArrayOf(radius, radius, radius, radius, 0f, 0f, 0f, 0f), Path.Direction.CCW)

        outerPath.reset()
        outerPath.addRect(bounds.left.toFloat(), bounds.top.toFloat(), bounds.right.toFloat(), bounds.bottom.toFloat(), Path.Direction.CCW)
        outerPath.op(path, Path.Op.XOR)
    }

    override fun getPadding(padding: Rect): Boolean {
        padding.set(this.padding)
        return !padding.isEmpty
    }

    override fun setBounds(left: Int, top: Int, right: Int, bottom: Int) {
        super.setBounds(left, top, right, bottom)
        updateBounds(bounds)
    }

    override fun setBounds(bounds: Rect) {
        super.setBounds(bounds)
        updateBounds(bounds)
    }

    override fun draw(canvas: Canvas) {
        paint.color = outerColor
        canvas.drawPath(outerPath, paint)

        paint.color = innerColor
        canvas.drawPath(path, paint)
    }

    override fun setAlpha(alpha: Int) {
    }

    override fun getOpacity(): Int {
        return PixelFormat.TRANSLUCENT
    }

    override fun setColorFilter(colorFilter: ColorFilter?) {

    }

}