package com.teslacoilsw.launcher.icon

import android.graphics.Path
import com.teslacoilsw.cofs.Cofs
import com.teslacoilsw.cofs.CofsBase
import com.teslacoilsw.cofs.ConfigObjectFromString
import com.teslacoilsw.cofs.getCofsDelegate

interface CustomShapePath {
    fun getPath(): Path
    fun asPathString(): String
}

class PathParserCustomShapePath (private val pathString: String): CustomShapePath {
    override fun getPath(): Path = PathParser.createPathFromPathData(pathString)
    override fun asPathString() = pathString
}

@ConfigObjectFromString
data class CornerCustomShapePath (val topLeft: CornerConfig, val topRight: CornerConfig = topLeft, val bottomLeft: CornerConfig = topLeft, val bottomRight: CornerConfig = topLeft): CustomShapePath {
    override fun getPath(): Path {
        val p = Path()
        val tl = topLeft.strength.toFloat()
        val tr = topRight.strength.toFloat()
        val bl = bottomLeft.strength.toFloat()
        val br = bottomRight.strength.toFloat()
        p.moveTo(0f, tl)
        val sqX = .20f // Samsung is more like .25 or .30
        when (topLeft.type) {
            CornerType.SQUIRCLE ->
                p.cubicTo(0f, tl * sqX, tl * sqX, 0f, tl, 0f)
            CornerType.ROUND ->
                p.arcTo(0f, 0f, 2 * tl, 2 * tl, -180f, 90f, false)
            CornerType.CROP ->
                p.lineTo(tl, 0f)
        }

        p.lineTo(100f-tr, 0f)
        when (topRight.type) {
            CornerType.SQUIRCLE ->
                p.cubicTo(100f-tr*sqX, 0f, 100f, tr*sqX, 100f, tr)
            CornerType.ROUND ->
                p.arcTo(100 - 2 * tr, 0f, 100f, 2 * tr, -90f, 90f, false)
            CornerType.CROP ->
                p.lineTo(100f, tr)
        }

        p.lineTo(100f, 100f-br)
        when (bottomRight.type) {
            CornerType.SQUIRCLE ->
                p.cubicTo(100f, 100f-br*sqX, 100f-br*sqX, 100f, 100f-br, 100f)
            CornerType.ROUND ->
                p.arcTo(100 - 2 * br, 100 - 2 * br, 100f, 100f, 0f, 90f, false)
            CornerType.CROP ->
                p.lineTo(100f - br, 100f)
        }

        p.lineTo(bl, 100f)
        when (bottomLeft.type) {
            CornerType.SQUIRCLE ->
                p.cubicTo(bl*sqX, 100f, 0f, 100f-bl*sqX, 0f, 100f-bl)
            CornerType.ROUND ->
                p.arcTo(0f, 100 - 2 * bl, 2*bl, 100f, 90f, 90f, false)
            CornerType.CROP ->
                p.lineTo(0f, 100f - bl)
        }
        p.close()
        return p
    }

    companion object : Cofs<CornerCustomShapePath>, CofsBase<CornerCustomShapePath> by CornerCustomShapePath::class.getCofsDelegate() {
        override val DEFAULT = CornerCustomShapePath(CornerConfig.DEFAULT, CornerConfig.DEFAULT, CornerConfig.DEFAULT, CornerConfig.DEFAULT)
    }

    override fun asPathString() = "SHAPE:" + flattenToString(this)
}

enum class CornerType { CROP, ROUND, SQUIRCLE }

@ConfigObjectFromString
data class CornerConfig(val strength: Int, val type: CornerType) {
    companion object : Cofs<CornerConfig>, CofsBase<CornerConfig> by CornerConfig::class.getCofsDelegate() {
        @JvmStatic
        override val DEFAULT = CornerConfig(25, CornerType.SQUIRCLE)
    }

    override fun toString(): String = flattenToString(this)
}
