package com.teslacoilsw.launcher.icontheme

import android.content.ComponentName
import android.content.Context
import android.content.Intent
import android.graphics.Bitmap
import android.graphics.drawable.Drawable
import android.net.Uri
import android.text.TextUtils
import com.android.launcher3.IconCache
import com.teslacoilsw.cofs.Cofs
import com.teslacoilsw.cofs.CofsBase
import com.teslacoilsw.cofs.ConfigObjectFromString
import com.teslacoilsw.cofs.getCofsDelegate
import com.teslacoilsw.launcher.BuildConfig
import com.teslacoilsw.launcher.R
import org.icontheme.api.ThemeConfig

interface IconTheme {
    val lastUpdateTime: Long
    val packageName: String
    val calendarPackages: Array<String>
    fun getThemedIconBitmap(context: Context, cache: IconCache, cn: ComponentName): Bitmap?
    fun getThemedIconFromBase(context: Context, cn: ComponentName, baseIcon: Bitmap): Bitmap?

    fun canGenerateIcon(): Boolean
    fun hasIconFor(cn: ComponentName): Boolean

    fun isEmpty(): Boolean = false
}

enum class IconPackType constructor(private val action: String) {
    GO("com.gau.go.launcherex.theme"),
    NOVA("com.novalauncher.THEME"),
//    APEX("") {
//        override val resolveIntent: Intent?
//            get() = Intent(Intent.ACTION_MAIN).addCategory("com.anddoes.launcher.THEME")
//    },
    ADW("org.adw.launcher.icons.ACTION_PICK_ICON"),
    BUILTIN(""),
    ICONTHEMEORG("");

    open val resolveIntent: Intent?
        get() = if (!TextUtils.isEmpty(action))
            Intent(action)
        else
            null
}

@ConfigObjectFromString(paramsRequired = 3)
data class IconThemeConfig(val name: String, val type: IconPackType, val pkg: String, val id: String? = null) {
    companion object : Cofs<IconThemeConfig>, CofsBase<IconThemeConfig> by IconThemeConfig::class.getCofsDelegate() {
        @JvmStatic
        override val DEFAULT = IconThemeConfig("", IconPackType.BUILTIN, "")

        override fun parse(string: String, defaultValue: IconThemeConfig): IconThemeConfig {
            val d: CofsBase<IconThemeConfig> = IconThemeConfig::class.getCofsDelegate()
            var c = d.parse(string, defaultValue)
            if (c.type == IconPackType.BUILTIN) {
                if (c.pkg != BuildConfig.APPLICATION_ID)
                    c = c.copy(pkg = BuildConfig.APPLICATION_ID, id = c.pkg)
                if (c.id == null)
                    return DEFAULT
            }
            return c
        }
    }

    fun getIcon(context: Context): Drawable? {
        return if (type == IconPackType.BUILTIN) {
            if (id == "LOLLIPOP")
                context.getDrawable(R.drawable.ic_lptheme)
            else
                null
        } else {
            try {
                context.applicationContext.packageManager.getApplicationIcon(pkg)
            } catch (e: Exception) {
                null
            }
        }
    }

    fun getIconTheme(context: Context, iconDpi: Int): IconTheme {
        if (this.type == IconPackType.BUILTIN && id == null)
            return ICON_THEME_EMPTY

        return when (type) {
            IconPackType.ADW -> ADWIconPack(context, pkg)
            IconPackType.GO -> XmlIconPack(context, pkg, iconDpi)
            IconPackType.NOVA -> XmlIconPack(context, pkg, iconDpi)
            IconPackType.BUILTIN -> {
                if (name.isBlank())
                    ICON_THEME_EMPTY
                else
                    IconThemeLollipop(context)
            }
            IconPackType.ICONTHEMEORG -> {
                ThemeConfig.createFromUri(context.contentResolver, context.packageManager, Uri.parse("content://$id"))
                        ?.let {
                            IconThemeOrgIconTheme(it)
                        } ?: ICON_THEME_EMPTY
            }
        }
    }
}

object ICON_THEME_EMPTY : IconTheme {
    override val lastUpdateTime
        get() = 0L
    override val packageName: String
        get() = ""
    override val calendarPackages: Array<String>
        get() = emptyArray()

    override fun getThemedIconBitmap(context: Context, cache: IconCache, cn: ComponentName): Bitmap? = null

    override fun getThemedIconFromBase(context: Context, cn: ComponentName, baseIcon: Bitmap): Bitmap? = null

    override fun canGenerateIcon(): Boolean = false

    override fun hasIconFor(cn: ComponentName): Boolean = false

    override fun isEmpty(): Boolean = true
}