package com.teslacoilsw.launcher.iteminfo

import android.content.ComponentName
import android.content.Intent
import android.graphics.Bitmap
import com.android.launcher3.LauncherSettings
import com.android.launcher3.util.ContentWriter

sealed class ItemIcon(val typeId: Int) {
    abstract fun write(writer: ContentWriter, info: ItemInfoWithIcon)
}

class ItemIconResource(val resource: Intent.ShortcutIconResource) : ItemIcon(LauncherSettings.Favorites.ICON_TYPE_RESOURCE) {
    constructor(pkg: String, resName: String) : this(Intent.ShortcutIconResource()
            .apply { packageName = pkg; resourceName = resName })
    override fun write(writer: ContentWriter, info: ItemInfoWithIcon) {
        if (!info.usingFallbackIcon && !info.isNovaIcon)
            writer.putIcon(info.iconBitmap, info.user)
        else
            writer.putIcon(null, info.user)

        writer.put(LauncherSettings.BaseLauncherColumns.ICON_TYPE, typeId)
        writer.put(LauncherSettings.BaseLauncherColumns.ICON_PACKAGE,
                    resource.packageName)
        writer.put(LauncherSettings.BaseLauncherColumns.ICON_RESOURCE,
                    resource.resourceName)
    }
}

class ItemIconUserBitmap(val bmp: Bitmap) : ItemIcon(LauncherSettings.Favorites.ICON_TYPE_USER_BITMAP) {
    override fun write(writer: ContentWriter, info: ItemInfoWithIcon) {
        if (info.iconSanifyScale != 0.0f && info.iconSanifyScale != 1.0f) {
            writer.put(LauncherSettings.BaseLauncherColumns.ICON_SANIFY_SCALE, info.iconSanifyScale)
            info.iconSanifyScale = 0.0f
        } else {
            writer.putNull(LauncherSettings.BaseLauncherColumns.ICON_SANIFY_SCALE)
            writer.put(LauncherSettings.BaseLauncherColumns.ICON_TYPE, typeId)
            writer.putIcon(bmp, info.user)
        }
    }
}

class ItemIconAllApps(val component: ComponentName) : ItemIcon(LauncherSettings.Favorites.ICON_TYPE_ALLAPPS) {
    override fun write(writer: ContentWriter, info: ItemInfoWithIcon) {
        writer.put(LauncherSettings.BaseLauncherColumns.ICON_TYPE, typeId)
        writer.putNull(LauncherSettings.BaseLauncherColumns.ICON_PACKAGE)
        writer.put(LauncherSettings.BaseLauncherColumns.ICON_RESOURCE,
                component.flattenToShortString())
        writer.putIcon(null, null)
    }

}

class ItemIconUpdatableBitmap(val bmp: Bitmap) : ItemIcon(LauncherSettings.Favorites.ICON_TYPE_UPDATABLE_BITMAP) {
    override fun write(writer: ContentWriter, info: ItemInfoWithIcon) {
        if (info.iconSanifyScale != 0.0f && info.iconSanifyScale != 1.0f) {
            writer.put(LauncherSettings.BaseLauncherColumns.ICON_SANIFY_SCALE, info.iconSanifyScale)
            info.iconSanifyScale = 0.0f
        } else {
            writer.putNull(LauncherSettings.BaseLauncherColumns.ICON_SANIFY_SCALE)
            writer.put(LauncherSettings.BaseLauncherColumns.ICON_TYPE, typeId)
            writer.putIcon(bmp, info.user)
        }
    }
}

/* Write


if (customIcon) {
            if (iconSanifyScale != 0.0f && iconSanifyScale != 1.0f) {
                writer.put(LauncherSettings.BaseLauncherColumns.ICON_SANIFY_SCALE, iconSanifyScale);
                iconSanifyScale = 0.0f;
            } else {
                writer.putNull(LauncherSettings.BaseLauncherColumns.ICON_SANIFY_SCALE);
                writer.put(LauncherSettings.BaseLauncherColumns.ICON_TYPE,
                        LauncherSettings.BaseLauncherColumns.ICON_TYPE_USER_BITMAP);
                writer.putIcon(iconBitmap, user);
            }
        } else if (iconResource != null) {
            if (!usingFallbackIcon && !isNovaIcon())
                writer.putIcon(iconBitmap, user);
            else
                writer.putIcon(null, user);

            writer.put(LauncherSettings.BaseLauncherColumns.ICON_TYPE,
                    LauncherSettings.BaseLauncherColumns.ICON_TYPE_RESOURCE);
            if (iconResource != null) {
                writer.put(LauncherSettings.BaseLauncherColumns.ICON_PACKAGE,
                        iconResource.packageName);
                writer.put(LauncherSettings.BaseLauncherColumns.ICON_RESOURCE,
                        iconResource.resourceName);
            }
        } else if (iconAllAppsCn != null) {
            writer.put(LauncherSettings.BaseLauncherColumns.ICON_TYPE,
                    LauncherSettings.BaseLauncherColumns.ICON_TYPE_ALLAPPS);
            writer.putNull(LauncherSettings.BaseLauncherColumns.ICON_PACKAGE);
            writer.put(LauncherSettings.BaseLauncherColumns.ICON_RESOURCE,
                    iconAllAppsCn.flattenToShortString());
            writer.putIcon(null, null);
        } else {
            writer.putIcon(null, null);
            writer.put(LauncherSettings.BaseLauncherColumns.ICON_TYPE,
                    LauncherSettings.BaseLauncherColumns.ICON_TYPE_RESOURCE);
        }
 */
