package com.teslacoilsw.launcher.onboarding

import android.content.Context
import android.content.SharedPreferences
import android.graphics.Color
import com.afollestad.materialdialogs.MaterialDialog
import com.afollestad.materialdialogs.StackingBehavior
import com.teslacoilsw.launcher.R
import com.teslacoilsw.launcher.daynight.DayNightContextWrapper
import com.teslacoilsw.launcher.preferences.COLOR_AUTO
import com.teslacoilsw.launcher.preferences.NightMode
import com.teslacoilsw.launcher.preferences.Pref3
import com.teslacoilsw.launcher.preferences.put
import com.teslacoilsw.launcher.util.MaterialColors
import io.reactivex.Observable

enum class ThemeOption(private val labelResId: Int, val previewId: Int, val prefEnum: NightMode) : WizardOption, PermissionOption {
    LIGHT(R.string.preset_light, R.drawable.presets_drawer_immersive, NightMode.NEVER),
    DARK(R.string.preset_dark, R.drawable.presets_drawer_immersive_dark, NightMode.ALWAYS),
    AUTO(R.string.auto, R.drawable.presets_drawer_immersive, NightMode.AUTO) {
        override val permissionsRequested: Array<out String> = arrayOf(android.Manifest.permission.ACCESS_COARSE_LOCATION, android.Manifest.permission.ACCESS_FINE_LOCATION)
        override val permissionsRequired: Boolean = false
    }
    ;

    override fun getLabel(context: Context) = context.getString(labelResId)!!
    override val permissionRequestCode: Int = name.hashCode() and 0xFFFF
    override val permissionsRequested: Array<out String> = arrayOf()
}

class NightModeThemePage : SimpleWizardPage<ThemeOption>() {
    override fun getTitle(context: Context): String = context.getString(R.string.preset_overall_theme)

    override fun save(context: Context, editor: SharedPreferences.Editor) {
        val isDark = currentOption == ThemeOption.DARK
        editor
            .put(Pref3.drawer_background_color, if (isDark) MaterialColors.NIGHT_THEME_BACKGROUND else Color.WHITE)
            .put(Pref3.drawer_card_color, if (isDark) MaterialColors.NIGHT_THEME_BACKGROUND else Color.WHITE)
            .put(Pref3.night_mode, currentOption.prefEnum)
            .put(Pref3.drawer_cellspecs, Pref3.drawer_cellspecs.value.copy(labelColor = COLOR_AUTO))
            .put(Pref3.folder_cellspecs, Pref3.drawer_cellspecs.value.copy(labelColor = COLOR_AUTO))
            .put(Pref3.folder_window_config, Pref3.folder_window_config.value.copy(rawBackgroundColor = if (isDark) MaterialColors.NIGHT_THEME_BACKGROUND else Color.WHITE))
            .put(Pref3.qsb_style, Pref3.qsb_style.value.createWithBackgroundColor(if (isDark) MaterialColors.NIGHT_THEME_BACKGROUND else Color.WHITE))

        if (isDark)
            editor.put(Pref3.night_mode_auto_drawer_icon, true)
    }

    override val options: Observable<WizardOptions<ThemeOption>> = Observable.just(WizardOptions(ThemeOption.LIGHT, listOf(ThemeOption.LIGHT, ThemeOption.DARK, ThemeOption.AUTO)))

    override fun updatePreview() {
        binding.preview.setImageResource(currentOption.previewId)
    }

    override fun onOptionSelected(selectedOption: ThemeOption): Boolean {
        if (selectedOption == ThemeOption.AUTO) {
            if (!selectedOption.hasPermissions(requireContext())) {
                MaterialDialog.Builder(DayNightContextWrapper(requireContext(), R.style.Nova_DayNight))
                        .title(R.string.use_location_access)
                        .content(R.string.nightmode_use_location_access_long)
                        .stackingBehavior(StackingBehavior.ALWAYS)
                        .autoDismiss(true)
                        .negativeText(R.string.no_thanks)
                        .positiveText(R.string.use_location)
                        .onPositive { _, _ ->
                            requestPermissions(ThemeOption.AUTO.permissionsRequested, ThemeOption.AUTO.permissionRequestCode)
                        }
                        .show()
            }
            return true
        }
        return super.onOptionSelected(selectedOption)
    }
}