package com.teslacoilsw.launcher.onboarding

import android.content.Context
import android.content.SharedPreferences
import com.teslacoilsw.launcher.R
import com.teslacoilsw.launcher.preferences.Pref
import com.teslacoilsw.launcher.preferences.Pref3
import com.teslacoilsw.launcher.preferences.put
import io.reactivex.Observable

enum class DrawerActionOption(private val labelResId: Int, val previewId: Int) : WizardOption {
    BUTTON(R.string.preset_dock_icon, R.drawable.presets_button),
    SWIPE(R.string.preset_swipe_up, R.drawable.presets_swipe),
    ;

    override fun getLabel(context: Context) = context.getString(labelResId)
}

class SwipeDrawerPage : SimpleWizardPage<DrawerActionOption>() {
    override fun getTitle(context: Context): String = context.getString(R.string.preset_open_app_drawer_action)

    override fun save(context: Context, editor: SharedPreferences.Editor) {
        val isSwipe = currentOption == DrawerActionOption.SWIPE
        editor.putBoolean(Pref3.drawer_swipe_to_open.key, isSwipe)
            .putBoolean(Pref3.drawer_swipe_indicator.key, isSwipe)
            .put(Pref3.desktop_page_indicator, if (isSwipe) Pref.Key.ScrollIndicatorStyle.LINE else Pref.Key.ScrollIndicatorStyle.CIRCLE)
            .putBoolean(Pref.Key.PRESET_ADD_ALLAPPS, !isSwipe)
    }

    override val options: Observable<WizardOptions<DrawerActionOption>> = Observable.just(WizardOptions(DrawerActionOption.SWIPE, listOf(DrawerActionOption.SWIPE, DrawerActionOption.BUTTON)))

    override fun updatePreview() {
        binding.preview.setImageResource(currentOption.previewId)
    }
}