package com.teslacoilsw.launcher.onboarding

import android.content.Intent
import android.os.Bundle
import android.preference.PreferenceManager
import androidx.appcompat.app.AppCompatActivity
import com.android.launcher3.LauncherAppState
import com.teslacoilsw.kotlin.bindContentView
import com.teslacoilsw.launcher.BuildConfig
import com.teslacoilsw.launcher.NovaLauncher
import com.teslacoilsw.launcher.NovaShortcutHandler
import com.teslacoilsw.launcher.R
import com.teslacoilsw.launcher.databinding.ActivityOnboardingBinding
import com.teslacoilsw.launcher.preferences.Pref
import io.reactivex.disposables.Disposable
import timber.log.Timber

abstract class WizardActivity : AppCompatActivity() {
    private var currentPage: Int = 0
    abstract val wizardPages : List<WizardPage>
    private lateinit var binding: ActivityOnboardingBinding
    private var finishOnNextSubscription: Disposable? = null
    private var finishOnNext = false

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = bindContentView(R.layout.activity_onboarding)
        binding.previous.setOnClickListener {
            if (currentPage == 0) {
                PreferenceManager.getDefaultSharedPreferences(this).edit()
                        .putBoolean("did_quickstart", true)
                        .apply()
                finish()
            } else {
                selectPage(currentPage - 1)
            }
        }
        binding.next.setOnClickListener {
            if (BuildConfig.DEBUG)
                Timber.v("onClick: $it")

            if (currentPage == wizardPages.lastIndex || finishOnNext) {
                NovaLauncher.sRecreateOnResume = true
                val editor = PreferenceManager.getDefaultSharedPreferences(this).edit()
                editor.putBoolean("did_quickstart", true)
                for (i in 0..currentPage) {
                    wizardPages[i].save(this, editor)
                }
                editor.apply()

                NovaLauncher.sRecreateOnResume = true
                LauncherAppState.getInstance().resetDynamicGrid()
                Pref.Value.init(this, Pref.Value.mPrefs)
                LauncherAppState.forceModelReload()

                val intent = Intent(NovaShortcutHandler.ACTION_FINISH)
                intent.component = NovaShortcutHandler.COMPONENT_NAME
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_TASK_ON_HOME)
                try {
                    startActivity(intent)
                } catch (ex: Exception) {
                    ex.printStackTrace()
                }


                finish()
            } else {
                selectPage(currentPage + 1)
            }
        }
        selectPage(0)

    }

    override fun onBackPressed() {
        binding.previous.callOnClick()
    }

    fun selectPage(newPage: Int) {
        if (BuildConfig.DEBUG)
            Timber.v("selectPage: $newPage")

        binding.previous.setText( if (newPage == 0) R.string.preset_set_defaults else R.string.preset_previous)
        finishOnNextSubscription?.dispose()
        currentPage = newPage
        finishOnNextSubscription = wizardPages[currentPage].finishOnNextObservable.subscribe {
            finishOnNext = it
            binding.next.setText(if (newPage == wizardPages.lastIndex || it == true) R.string.preset_apply else R.string.preset_next)
        }
        supportFragmentManager.beginTransaction()
            .replace(R.id.page, wizardPages[currentPage])
            .commit()
    }

    override fun onDestroy() {
        super.onDestroy()
        finishOnNextSubscription?.dispose()
    }
}