package com.teslacoilsw.launcher.onboarding

import android.content.Context
import android.util.AttributeSet
import android.view.LayoutInflater
import androidx.core.view.children
import com.google.android.material.chip.Chip
import com.google.android.material.chip.ChipGroup
import com.teslacoilsw.launcher.R
import io.reactivex.Observable
import io.reactivex.disposables.Disposable

class WizardOptionChipGroup<T: WizardOption>(context: Context, attrs: AttributeSet, defStyle: Int) : ChipGroup(context, attrs, defStyle) {
    private var optionsSubscription: Disposable? = null
    public var optionsObseverable: Observable<Pair<T, List<T>>> = Observable.empty()
        set(value) {
            field = value
            optionsSubscription?.dispose()
            optionsSubscription = field.subscribe {
                removeAllViews()

                val inflater = LayoutInflater.from(context)
                for ((index, item) in it.second.withIndex()) {
                    val chip: Chip = inflater.inflate(R.layout.onboarding_chip_item, this, false) as Chip
                    chip.text = item.getLabel(context)
                    chip.tag = item
                    chip.id = index
                    chip.isChecked = item == it.first
                    addView(chip)
                }
            }
        }

    fun getSelectedOption() : T? {
        for (chip in children) {
            if ((chip as? Chip)?.isChecked == true) {
                return chip.tag as T
            }
        }
        return null
    }

}