package com.teslacoilsw.launcher.preferences

import android.content.Intent
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.graphics.Canvas
import android.graphics.Color
import android.os.Bundle
import android.view.View
import android.widget.RadioButton
import com.android.launcher3.BubbleTextView
import com.android.launcher3.FastBitmapDrawable
import com.android.launcher3.FolderIcon
import com.teslacoilsw.kotlin.lazy1
import com.teslacoilsw.kotlin.tryCatch
import com.teslacoilsw.launcher.FolderIconHelper
import com.teslacoilsw.launcher.NovaLauncher
import com.teslacoilsw.launcher.R
import com.teslacoilsw.launcher.icon.AdaptiveIconShape
import com.teslacoilsw.launcher.preferences.fancyprefs.FancySettingsActivity
import com.teslacoilsw.launcher.util.DpUtil
import com.teslacoilsw.launcher.widget.DumbRadioGrid
import com.teslacoilsw.shared.util.MathUtils
import kotlinx.android.synthetic.main.activity_folder_icon_settings.*
import timber.log.Timber
import java.io.File
import java.io.FileOutputStream
import java.io.IOException

private const val REQUEST_PICK_FOLDER_BACKGROUND = 100
class FolderIconSettingsActivity : FancySettingsActivity(R.layout.activity_folder_icon_settings) {
    private val folderIconHelper by lazy1 { FolderIconHelper(this) }
    private val shapeButtons by lazy1 { arrayOf(shapeAdaptive, shapeCircle, shapeSquircle, shapeRoundedSquare, shapeSquare, shapeTeardrop, shapePlatform, shapeCustom) }
    private val appIconSize by lazy1 { DpUtil.getPixelSize(72) }
    private val folderIconSize by lazy1 { MathUtils.roundInt(FolderIcon.FOLDER_ICON_SCALE * appIconSize) }
    private val previewBitmap by lazy1 { Bitmap.createBitmap(folderIconSize, folderIconSize, Bitmap.Config.ARGB_8888) }
    private val previewCanvas = Canvas()
    private val previewItems by lazy1 { listOf(
            BubbleTextView(this).apply { applyCompoundDrawables(FastBitmapDrawable(BitmapFactory.decodeResource(resources, R.drawable.ic_lptheme_settings)).apply { setBounds(0, 0, appIconSize, appIconSize) }) },
            BubbleTextView(this).apply { applyCompoundDrawables(FastBitmapDrawable(BitmapFactory.decodeResource(resources, R.drawable.ic_lptheme_email)).apply { setBounds(0, 0, appIconSize, appIconSize) }) },
            BubbleTextView(this).apply { applyCompoundDrawables(FastBitmapDrawable(BitmapFactory.decodeResource(resources, R.drawable.ic_lptheme_clock)).apply { setBounds(0, 0, appIconSize, appIconSize) }) },
            BubbleTextView(this).apply { applyCompoundDrawables(FastBitmapDrawable(BitmapFactory.decodeResource(resources, R.drawable.ic_lptheme_sms)).apply { setBounds(0, 0, appIconSize, appIconSize) }) }
        ) }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        previewGroup.check(Pref3.folder_icon_config.value.preview.radioButtonId)
        bgGroup.check(Pref3.folder_icon_config.value.background.radioButtonId)

        val onChangeListener = object: DumbRadioGrid.OnCheckedChangeListener {
            override fun onCheckedChanged(group: DumbRadioGrid, checkedId: Int) {
                updatePreview()
            }
        }
        bgGroup.setOnCheckedChangeListener(object: DumbRadioGrid.OnCheckedChangeListener {
            override fun onCheckedChanged(group: DumbRadioGrid, checkedId: Int) {
                if (checkedId == Pref.Key.FolderBg.CUSTOM.radioButtonId) {
                    val pickerIntent = Intent(Intent.ACTION_GET_CONTENT)
                    pickerIntent.type = "image/*"
                    startActivityForResult(Intent.createChooser(pickerIntent, resources.getString(R.string.select_file)), REQUEST_PICK_FOLDER_BACKGROUND)
                } else {
                    updatePreview()
                }
            }
        })
        previewGroup.setOnCheckedChangeListener(onChangeListener)

        shapeAdaptive.tag = AdaptiveIconShape.NONE
        shapeCircle.tag = AdaptiveIconShape.CIRCLE
        shapeSquircle.tag = AdaptiveIconShape.SQUIRCLE
        shapeRoundedSquare.tag = AdaptiveIconShape.ROUNDED_SQUARE
        shapeSquare.tag = AdaptiveIconShape.SQUARE
        shapeTeardrop.tag = AdaptiveIconShape.TEARDROP
        shapePlatform.tag = AdaptiveIconShape.FOLDER_PLATFORM
        shapeCustom.tag = AdaptiveIconShape.CUSTOM

        for (shapeButton in shapeButtons) {
            var shape = shapeButton.tag as AdaptiveIconShape
            if (shape == AdaptiveIconShape.NONE)
                shape = Pref3.adaptive_icon_shape.value

            if (shape != AdaptiveIconShape.CUSTOM)
                shapeButton.setCompoundDrawables(null, shape.createPreviewDrawable(this), null, null)
            if (shape == Pref3.folder_icon_config.value.shape)
                shapeButton.isChecked = true
        }

        shapeCustom.setOnClickListener {
            val pickerIntent = Intent(Intent.ACTION_GET_CONTENT)
            pickerIntent.type = "image/*"
            startActivityForResult(Intent.createChooser(pickerIntent, resources.getString(R.string.select_file)), REQUEST_PICK_FOLDER_BACKGROUND)
        }

        shapeGroup.setOnCheckedChangeListener(object: DumbRadioGrid.OnCheckedChangeListener {
            override fun onCheckedChanged(group: DumbRadioGrid, checkedId: Int) {
                if (checkedId != R.id.shapeCustom) {
                    updatePreview()
                }
            }
        })

        clipToBounds.isChecked = Pref3.folder_icon_config.value._clipToBounds
        clipToBounds.onUserChanged = ::updatePreview

        colorButton.color = Pref3.folder_icon_config.value._iconColor
        colorButton.onUserChanged = ::updatePreview

        transparency.progress = 255 - Pref3.folder_icon_config.value.iconAlpha
        transparency.onUserChanged = ::updatePreview

        updatePreview()
    }

    override fun onPause() {
        super.onPause()
        if (Pref3.folder_icon_config.setIfChanged(folderIconConfigFromCurrent()))
            NovaLauncher.postRecreate()
    }

    private fun folderIconConfigFromCurrent(): FolderIconConfig {
//        val bg = tryCatch(Pref.Key.FolderBg.PEEPHOLE) { Pref.Key.FolderBg.valueOf(bgGroup.findViewById<View>(bgGroup.checkedRadioButtonId)?.tag as String? ?: Pref.Key.FolderBg.PEEPHOLE.name) }
        val fg = tryCatch(Pref.Key.FolderPreview.GRID) { Pref.Key.FolderPreview.valueOf(previewGroup.findViewById<View>(previewGroup.checkedRadioButtonId)?.tag as String? ?: Pref.Key.FolderPreview.GRID.name) }
        val shape = shapeGroup.findViewById<View>(shapeGroup.checkedRadioButtonId)?.tag as AdaptiveIconShape? ?: AdaptiveIconShape.NONE
        val alpha = 255 - transparency.progress
        return FolderIconConfig(shape, fg, clipToBounds.isChecked, colorButton.color, alpha)
    }

    private fun updatePreview(old: Any = Unit, new: Any = Unit) {
        val config = folderIconConfigFromCurrent()
        folderIconHelper.setConfig(this, config)
        previewBitmap.eraseColor(Color.TRANSPARENT)
        previewCanvas.setBitmap(previewBitmap)

        folderIconHelper.computePreviewDrawingParams(folderIconSize, appIconSize, folderIconSize)
        folderIconHelper.drawFolderIcon(previewCanvas, previewItems, null, true,false, null, Color.TRANSPARENT, false)
        previewCanvas.setBitmap(null)
        previewIcon.applyCompoundDrawables(FastBitmapDrawable(previewBitmap).apply { setBounds(0, 0, folderIconSize, folderIconSize) })
        clipToBounds.isEnabled = config.canClipToBounds()
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        when (requestCode) {
            REQUEST_PICK_FOLDER_BACKGROUND -> {
                if (resultCode == RESULT_OK) {
                    val photoUri = data?.data
                    if (photoUri != null) {
                        try {
                            val cr = contentResolver
                            var inputStream = cr.openInputStream(photoUri)
                            val opts = BitmapFactory.Options()
                            opts.inJustDecodeBounds = true
                            BitmapFactory.decodeStream(inputStream, null, opts)
                            val ops2 = BitmapFactory.Options()
                            val width = folderIconSize
                            val w = opts.outWidth
                            val scale = Math.max(1, (w.toFloat() / width).toInt())
                            ops2.inSampleSize = scale
                            inputStream = cr.openInputStream(photoUri)
                            val bitmap = BitmapFactory.decodeStream(inputStream, null, ops2)
                            processBitmap(bitmap)
                        } catch (e: Exception) {
                            findViewById<RadioButton>(Pref.Key.FolderBg.DISABLED.radioButtonId).isChecked = true
                            e.printStackTrace()
                        }
                    } else {
                        findViewById<RadioButton>(Pref.Key.FolderBg.DISABLED.radioButtonId).isChecked = true
                    }
                } else {
                    updatePreview()
                }
                return
            }
        }

        super.onActivityResult(requestCode, resultCode, data)
    }

    private fun processBitmap(inBitmap: Bitmap?) {
        if (inBitmap == null) {
            shapeGroup.check(R.id.shapeAdaptive)
            return
        }
        var bitmap: Bitmap = inBitmap
        if (bitmap.width != folderIconSize || bitmap.height != folderIconSize)
            bitmap = Bitmap.createScaledBitmap(bitmap, folderIconSize, folderIconSize, true)

        val outFile = File(Pref.Key.FolderBg.getCustomImagePath(this))
        try {
            outFile.parentFile.mkdirs()
            val outputStream = FileOutputStream(outFile)
            outputStream.use {
                bitmap.compress(Bitmap.CompressFormat.PNG, 100, outputStream)
            }
            Pref.Key.FolderBg.CUSTOM.resetImageDrawable()
            updatePreview()
        } catch (ex: IOException) {
            Timber.tag("Nova.FolderPreference").e(ex, "Cannot open file: %s", outFile)
            shapeGroup.check(R.id.shapeAdaptive)
        }
    }
}
