package com.teslacoilsw.launcher.preferences

import android.content.ComponentName
import android.content.res.ColorStateList
import android.graphics.Bitmap
import android.graphics.Color
import android.graphics.Rect
import android.os.Build
import android.os.Bundle
import android.view.View
import android.view.ViewGroup
import com.android.launcher3.BubbleTextView
import com.android.launcher3.CellLayout
import com.android.launcher3.FastBitmapDrawable
import com.android.launcher3.Folder
import com.android.launcher3.LauncherAppState
import com.android.launcher3.ShortcutInfo
import com.teslacoilsw.kotlin.bind
import com.teslacoilsw.launcher.NovaLauncher
import com.teslacoilsw.launcher.R
import com.teslacoilsw.launcher.databinding.ActivityFolderWindowSettingsBinding
import com.teslacoilsw.launcher.getForegroundColorForBackground
import com.teslacoilsw.launcher.icontheme.IconThemeLollipop
import com.teslacoilsw.launcher.preferences.fancyprefs.FancySettingsActivity
import com.teslacoilsw.shared.colorpicker.ColorUtil
import java.util.*

class FolderWindowSettingsActivity : FancySettingsActivity(R.layout.activity_folder_window_settings) {
    private lateinit var binding : ActivityFolderWindowSettingsBinding
    val previewIcons = ArrayList<BubbleTextView>()

    private lateinit var config: FolderWindowConfig
    private val cellSpecs by lazy { LauncherAppState.getInstance().dynamicGrid.deviceProfile.folderCellSpecs.copyWithoutNightMode() }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = rootView.bind()

        var flags = View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN or View.SYSTEM_UI_FLAG_LAYOUT_STABLE
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M &&
                Pref3.notification_bar_transparency.value &&
                Pref3.notification_bar_dark_icons.value) {
            flags = flags or View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR
        }
        window.statusBarColor = Color.TRANSPARENT
        binding.folderPreview.root.systemUiVisibility = flags

        binding.root.setOnApplyWindowInsetsListener { _, insets ->
            (binding.back.layoutParams as ViewGroup.MarginLayoutParams).topMargin = insets.systemWindowInsetTop
            (binding.folderPreview.root as Folder).setInsets(Rect(0, insets.systemWindowInsetTop, 0, 0))
            insets
        }

        config = Pref3.folder_window_config.value
        binding.windowStyle.value = config.style.name
        binding.backgroundColor.color = config.rawBackgroundColor
        binding.transparency.progress = 255 - config.rawBackgroundAlpha
        binding.cornerRadius.progress = config.cornerRadiusDp
        binding.cardBackground.isChecked = config.cardBackground
        binding.showFolderName.isChecked = config.showFolderName
        binding.grid.value = config.rows to config.cols
        binding.scrollDirection.value = config.scrollDirection.name

        binding.windowStyle.onUserChanged = ::updatePreview
        binding.backgroundColor.onUserChanged = ::updatePreview
        binding.transparency.onUserChanged = ::updatePreview
        binding.cornerRadius.onUserChanged = ::updatePreview
        binding.cardBackground.onUserChanged = ::updatePreview
        binding.showFolderName.onUserChanged = ::updatePreview
        binding.grid.onUserChanged = ::updatePreview
        binding.scrollDirection.onUserChanged = ::updatePreview

        binding.folderPreview.folderName.apply {
            setText(getString(R.string.app_group_folder))
            isEnabled = false
        }
        binding.folderPreview.folderMenu.isEnabled = false
        binding.folderPreview.folderContent.setGridSize(4, 2)
        binding.folderPreview.folderContent.setupContentDimensions(Collections.emptyList(), 4)
        val views = ArrayList<View>()
        listOf("Calculator" to "com.android.calculator2/.Calculator", "Clock" to "com.android.deskclock/.DeskClock", "Email" to "com.android.email/.activity.Welcome", "Nova Settings" to "com.teslacoilsw.launcher/.preferences.SettingsActivity")
                .forEachIndexed{ index, tag ->
                    val v = layoutInflater.inflate(R.layout.application, binding.folderPreview.folderContent, false) as BubbleTextView
                    v.tag = tag.second
                    v.text = tag.first
                    val  lp = CellLayout.LayoutParams(index, 0, 1, 1)
                    previewIcons.add(v)
                    val info = ShortcutInfo()
                    info.intent = ShortcutInfo.makeMainIntent(ComponentName.unflattenFromString(tag.second))
                    v.tag = info
                    info.indexedRank = index
                    info.cellX = index.toFloat()
                    v.layoutParams = lp
                    views.add(binding.folderPreview.folderContent.addViewForRank(v, info, index)!!)
                }
        (binding.folderPreview.root as Folder).mItemsInvalidated = true
        binding.folderPreview.folderContent.setupContentDimensions(views, views.size)

        val cache = LauncherAppState.getInstance().iconCache
        var iconTheme = cache.iconTheme
        val defaultTheme = IconThemeLollipop(this)
        if (iconTheme.isEmpty()) {
            iconTheme = defaultTheme
        }
        for (i in 0 until previewIcons.size) {
            val tv = previewIcons[i]
            val cn = (tv.tag as ShortcutInfo).targetComponent
            var icon: Bitmap? = iconTheme.getThemedIconBitmap(this, cache, cn)
            if (icon == null) {
                icon = defaultTheme.getThemedIconBitmap(this, cache, cn)
            }
            tv.applyCellSpecs(cellSpecs)
            tv.setIcon(FastBitmapDrawable(icon))
        }

        updatePreview()
    }

    private fun updatePreview(old: Any = Unit, new: Any = Unit) {
        config = FolderWindowConfig(FolderWindowStyle.valueOf(binding.windowStyle.value), 255-binding.transparency.progress, binding.backgroundColor.color,binding.cornerRadius.progress, if (binding.cardBackground.isEnabled) binding.cardBackground.isChecked else config.rawCardBackground, binding.showFolderName.isChecked, binding.grid.value.first, binding.grid.value.second, FolderScrollDirection.valueOf(binding.scrollDirection.value))
        config.ignoreNightMode = true

        binding.grid.isEnabled = config.style == FolderWindowStyle.IMMERSIVE
        binding.cardBackground.isEnabled = config.style == FolderWindowStyle.IMMERSIVE
        binding.cardBackground.isChecked = config.cardBackground

        binding.cornerRadius.isEnabled = binding.cardBackground.isChecked

        cellSpecs.updateForBackgroundColor(ColorUtil.argb(config.backgroundAlpha, config.backgroundColor))
        (binding.folderPreview.root as Folder).setFolderWindowConfig(config, cellSpecs)

        for (i in 0 until previewIcons.size)
            previewIcons[i].applyCellSpecs(cellSpecs)

        binding.back.imageTintList =
                ColorStateList.valueOf(getForegroundColorForBackground(if (config.cardBackground) Color.BLACK else ColorUtil.argb(config.backgroundAlpha, config.backgroundColor)))
    }

    override fun onPause() {
        super.onPause()
        if (Pref3.folder_window_config.setIfChanged(config))
            NovaLauncher.postRecreate()
    }

}