package com.teslacoilsw.launcher.preferences

import android.annotation.SuppressLint
import android.content.ComponentName
import android.graphics.Bitmap
import android.graphics.Color
import android.graphics.LightingColorFilter
import android.graphics.Paint
import android.graphics.Typeface
import android.os.Build
import android.os.Bundle
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import android.widget.AdapterView
import android.widget.ArrayAdapter
import android.widget.ImageView
import android.widget.RelativeLayout
import android.widget.SeekBar
import android.widget.Spinner
import android.widget.TextView
import androidx.core.content.ContextCompat
import com.android.launcher3.BubbleTextView
import com.android.launcher3.DeviceProfile
import com.android.launcher3.FastBitmapDrawable
import com.android.launcher3.LauncherAppState
import com.android.launcher3.LauncherModel
import com.teslacoilsw.kotlin.dp
import com.teslacoilsw.kotlin.get
import com.teslacoilsw.kotlin.lazy1
import com.teslacoilsw.kotlin.size
import com.teslacoilsw.launcher.CellIconSizeSpecs
import com.teslacoilsw.launcher.CellLayoutType
import com.teslacoilsw.launcher.CellSpecConfig
import com.teslacoilsw.launcher.NovaLauncher
import com.teslacoilsw.launcher.R
import com.teslacoilsw.launcher.dock.DockDrawable
import com.teslacoilsw.launcher.icontheme.IconThemeLollipop
import com.teslacoilsw.launcher.preferences.fancyprefs.FancyPrefCheckableView
import com.teslacoilsw.launcher.preferences.fancyprefs.FancyPrefColorView
import com.teslacoilsw.launcher.preferences.fancyprefs.FancyPrefDisabledSegmentSeekBarView
import com.teslacoilsw.launcher.preferences.fancyprefs.FancySettingsActivity
import com.teslacoilsw.launcher.widget.TypefaceHelper
import com.teslacoilsw.shared.colorpicker.ColorUtil
import com.teslacoilsw.shared.util.MathUtils
import kotterknife.bindView
import kotlin.math.roundToInt

const val DOCK_SMALL_ICON_SCALE = 80
class IconLayoutSettingsActivity : FancySettingsActivity(R.layout.activity_iconlayout) {
    private val previewIcons: ViewGroup by bindView(R.id.preview_icons)
    private val iconScale: FancyPrefDisabledSegmentSeekBarView by bindView(R.id.icon_scale)
    private val iconScaleLink: FancyPrefCheckableView by bindView(R.id.icon_scale_link)
    private val hint: TextView by bindView(R.id.hint)
    private val labelVisible: FancyPrefCheckableView by bindView(R.id.label_visible)
    private val labelFamily:Spinner by bindView(R.id.label_family)
    private val labelColor: FancyPrefColorView by bindView(R.id.label_color)
    private val labelShadow: FancyPrefCheckableView by bindView(R.id.label_shadow)
    private val labelSingleLine: FancyPrefCheckableView by bindView(R.id.label_single_line)
    private val labelSize: SeekBar by bindView(R.id.label_size_seekbar)
    private val labelFamilyHint: TextView by bindView(R.id.label_family_hint)
    private val labelSizeHint: TextView by bindView(R.id.label_size_hint)
    private val actionBarBack: ImageView by bindView(R.id.back)

    private val novaSettingsIcon: BubbleTextView by bindView(R.id.preview_icon_nova_settings)

    private val displayMetrics by lazy1 { resources.displayMetrics }
    private val cellSpecs by lazy1 { CellIconSizeSpecs(type, cellSpecsPref.value.copy()) }

    private var labelSizeDp = 10f
    private var labelFamilyValue = ""

    private lateinit var cellSpecsPref: Pref3.Pref<CellSpecConfig>
    private var backgroundColor: Int = Color.TRANSPARENT
    private val type: CellLayoutType by lazy1 { CellLayoutType.valueOf(intent.getStringExtra("category")) }

    public override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        overridePendingTransition(0, 0)

        if (intent == null || !intent.hasExtra("category")) {
            finish()
            return
        }

        LauncherAppState.getInstance().initDynamicGrid(this, windowManager.defaultDisplay)

        actionBarBack.setOnClickListener {
            onBackPressed()
        }

        val dp16 = displayMetrics.dp(16)

        cellSpecsPref = when (type) {
            CellLayoutType.DRAWER -> Pref3.drawer_cellspecs
            CellLayoutType.DESKTOP -> Pref3.desktop_cellspecs
            CellLayoutType.DOCK -> Pref3.dock_cellspecs
            CellLayoutType.FOLDER -> Pref3.folder_cellspecs
        }

        when (type) {
            CellLayoutType.DRAWER -> {
                val baseBgColor: Int = if (Pref3.drawer_cards.value) {
                    Pref2._Drawer.DRAWER_BG_FOR_CARDS
                } else {
                    Pref3.drawer_background_color.value
                }

                findViewById<View>(R.id.previewFrame).setBackgroundColor(baseBgColor)
                if (Pref3.drawer_cards.value) {
                    backgroundColor = ColorUtil.argb(0xff, Pref3.drawer_card_color.value)
                    val cf = LightingColorFilter(backgroundColor, Color.rgb(0, 0, 0))
                    val d = ContextCompat.getDrawable(this, R.drawable.quantum_panel)!!
                    d.colorFilter = cf
                    d.alpha = Color.alpha(Pref3.drawer_card_color.value)
                    previewIcons.background = d
                    previewIcons.setPadding(dp16, 0, dp16, 0)
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                        window.statusBarColor = ColorUtil.compositeColors(0x3d000000, Pref2.get().drawer.drawerBackground)
                    }
                } else {
                    backgroundColor = ColorUtil.compositeColors(Pref2.get().drawer.drawerBackground, 0xFF424242.toInt())
                    val lightBg = ColorUtil.calculateContrast(Color.WHITE, backgroundColor) < 1.5
                    if (lightBg)
                        actionBarBack.setColorFilter(Color.BLACK)
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                        window.statusBarColor = ColorUtil.compositeColors(0x3d000000, Pref2.get().drawer.drawerBackground)
                    }
                }
            }
            CellLayoutType.FOLDER -> {
                backgroundColor = ColorUtil.argb(0xff, Pref3.folder_window_config.value.backgroundColor)
                val cf = LightingColorFilter(backgroundColor, Color.rgb(0, 0, 0))
                val d = ContextCompat.getDrawable(this, R.drawable.quantum_panel)!!
                d.colorFilter = cf
                d.alpha = Pref3.folder_window_config.value.backgroundAlpha
                previewIcons.background = d
                previewIcons.setPadding(dp16, 0, dp16, 0)
                //            val lp = previewIcons.layoutParams as ViewGroup.MarginLayoutParams
                //            val dp48 = displayMetrics.dp(48)
                //            lp.setMargins(dp48, displayMetrics.dp(32), dp48, displayMetrics.dp(12))
                //            previewIcons.layoutParams = lp
            }
            CellLayoutType.DOCK -> {
                if (Pref.Value.dock_background.enabled) {
                    val mDockDrawable = DockDrawable(resources)
                    Pref.Value.dock_background.setupDrawable(this, mDockDrawable, true)
                    previewIcons.background = mDockDrawable
                    val lp = previewIcons.layoutParams as RelativeLayout.LayoutParams
                    lp.addRule(RelativeLayout.CENTER_VERTICAL, 1)
                    backgroundColor = Pref.Value.dock_background.color
                }
            }
            CellLayoutType.DESKTOP -> {
                    iconScaleLink.visibility = View.GONE
            }
        }

        iconScale.progress = (cellSpecsPref.value.iconScale*100).roundToInt()

        labelVisible.isChecked = cellSpecsPref.value.labelVisible
        labelFamilyValue = cellSpecsPref.value.labelFont
        labelSizeDp = cellSpecsPref.value.labelSizeSp
        labelColor.color = cellSpecsPref.value.labelColor
        labelShadow.isChecked = cellSpecsPref.value.labelShadow
        labelSingleLine.isChecked = cellSpecsPref.value.labelSingleLine

        /* This fixes mTopDrawable which is not normally populated by xml */
        val cache = LauncherAppState.getInstance().iconCache
        var iconTheme = cache.iconTheme
        val defaultTheme = IconThemeLollipop(this)
        if (iconTheme.isEmpty()) {
            iconTheme = defaultTheme
        }
        for (i in 0 until previewIcons.size) {
            val tv = previewIcons[i] as BubbleTextView
            var icon: Bitmap? = iconTheme.getThemedIconBitmap(this, cache, ComponentName.unflattenFromString(tv.tag as String))
            if (icon == null) {
                icon = defaultTheme.getThemedIconBitmap(this, cache, ComponentName.unflattenFromString(tv.tag as String))
            }
            val d = FastBitmapDrawable(icon)
            tv.setCompoundDrawables(null, d, null, null)
        }

        iconScale.onUserChanged = ::updateSpecs
        iconScaleLink.isChecked = cellSpecs.config.matchDesktopSize
        iconScaleLink.onUserChanged = { _, _ ->
            if (iconScaleLink.isChecked)
                iconScale.progress = (LauncherAppState.getInstance().dynamicGrid.deviceProfile.workspaceCellSpecs.config.iconScale*100f).roundToInt()
            updateSpecs()
        }
        labelVisible.onUserChanged = ::updateSpecs
        labelShadow.onUserChanged = ::updateSpecs
        labelColor.onUserChanged = ::updateSpecs
        labelSingleLine.onUserChanged = ::updateSpecs

        val labelFamilyValues = resources.getStringArray(R.array.pref_label_fontfamily_values)
        val adapter = object : ArrayAdapter<CharSequence>(this,
                R.layout.simple_spinner_item,
                resources.getTextArray(R.array.pref_label_fontfamily)) {
            override fun getDropDownView(position: Int, convertView: View?, parent: ViewGroup): View {
                val view = super.getDropDownView(position, convertView, parent) as TextView
                view.typeface = TypefaceHelper.create(context, labelFamilyValues[position], Typeface.NORMAL)
                return view
            }

            override fun getView(position: Int, convertView: View?, parent: ViewGroup): View {
                val view = super.getView(position, convertView, parent) as TextView
                view.typeface = TypefaceHelper.create(context, labelFamilyValues[position], Typeface.NORMAL)
                return view
            }
        }
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
        labelFamily.adapter = adapter
        for (i in labelFamilyValues.indices) {
            val v = labelFamilyValues[i]
            if (labelFamilyValue == v) {
                labelFamily.setSelection(i)
                break
            }
        }

        labelFamily.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {
            override fun onItemSelected(parent: AdapterView<*>, view: View?, position: Int, id: Long) {
                val value = labelFamilyValues[position]
                labelFamilyValue = value
                updateSpecs()
            }

            override fun onNothingSelected(parent: AdapterView<*>) {

            }
        }

        labelSize.progress = when {
            labelSizeDp <= 11 -> 0
            labelSizeDp <= 12 -> 1
            labelSizeDp <= 13 -> 2
            labelSizeDp <= 14.4f -> 3
            labelSizeDp <= 16 -> 4
            labelSizeDp <= 18 -> 5
            labelSizeDp <= 21 -> 6
            else -> 2
        }
        labelSize.setOnSeekBarChangeListener(object : SeekBar.OnSeekBarChangeListener {
            override fun onProgressChanged(seekBar: SeekBar, progress: Int, fromUser: Boolean) {
                when (progress) {
                    0 -> labelSizeDp = 11f
                    1 -> labelSizeDp = 12f
                    2 -> labelSizeDp = 13f
                    3 -> labelSizeDp = 14.4f
                    4 -> labelSizeDp = 16f
                    5 -> labelSizeDp = 18f
                    6 -> labelSizeDp = 21f
                }
                updateSpecs()
            }

            override fun onStartTrackingTouch(seekBar: SeekBar) {

            }

            override fun onStopTrackingTouch(seekBar: SeekBar) {

            }
        })

        updateSpecs()
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when (item.itemId) {
            android.R.id.home -> {
                onBackPressed()
                return true
            }
        }

        return super.onOptionsItemSelected(item)
    }

    private val mTextPaint = Paint()
    private fun updateLayoutForDockBackground() {
        val drawablePadding = resources.getDimensionPixelSize(R.dimen.dynamic_grid_icon_drawable_padding)
        val edgeMarginPx = resources.getDimensionPixelSize(R.dimen.dynamic_grid_edge_margin)
        val textPaint = mTextPaint
        textPaint.textSize = cellSpecs.iconTextSizePx.toFloat()
        val fm = textPaint.fontMetrics
        val textSize = Math.ceil((if (cellSpecs.config.labelSingleLine) 1f else DeviceProfile.TWO_LINE_MULTIPLER) * Math.ceil((fm.bottom - fm.top).toDouble())).toInt()
        val hotseatCellHeightPx = cellSpecs.iconSizePx + if (cellSpecs.config.labelVisible) drawablePadding + textSize else 0
        val hotseatBarHeightPx = hotseatCellHeightPx + 4 * edgeMarginPx

        previewIcons.layoutParams.height = hotseatBarHeightPx
    }

    private fun updateSpecs(old: Any = Unit, new: Any = Unit) {
        val labelChecked = labelVisible.isChecked
        labelFamily.isEnabled = labelChecked
        labelSize.isEnabled = labelChecked
        labelColor.isEnabled = labelChecked
        labelShadow.isEnabled = labelChecked
        labelSingleLine.isEnabled = labelChecked
        labelSizeHint.isEnabled = labelChecked
        labelFamilyHint.isEnabled = labelChecked

        iconScale.isEnabled = iconScaleLink.visibility != View.VISIBLE || !iconScaleLink.isChecked

        val baseSize = resources.getDimensionPixelSize(R.dimen.app_icon_size)

        cellSpecs.config = CellSpecConfig(
            rawIconScale = iconScale.progress/100f,
            matchDesktopSize = iconScaleLink.isChecked,
            labelVisible = labelVisible.isChecked,
            labelColor = labelColor.color,
            labelShadow = labelShadow.isChecked,
            labelFont = labelFamilyValue,
            labelSingleLine = labelSingleLine.isChecked,
            labelSizeSp = labelSizeDp
        )

        @SuppressLint("SetTextI18n")
        if (cellSpecs.config.labelSingleLine) {
            novaSettingsIcon.text = "Nova Settings"
        } else {
            novaSettingsIcon.text = "Nova\nSettings"
        }

        cellSpecs.applyIconSize(displayMetrics, baseSize, if (cellSpecs.config.labelVisible) displayMetrics.dp(4) else 0, 1.0f)

        val maxScale = LauncherAppState.getInstance().dynamicGrid.deviceProfile.getMaxIconScale(this, type, cellSpecs)
        val scale = Math.min(maxScale, iconScale.progress / 100f)

        cellSpecs.applyIconSize(displayMetrics, baseSize, if (cellSpecs.config.labelVisible) displayMetrics.dp(4) else 0, scale)
        cellSpecs.updateForBackgroundColor(backgroundColor)

        val maxScaleInt = Math.min(150, MathUtils.ceilInt(maxScale * 100))
        iconScale.effectiveMax = maxScaleInt

        if (maxScaleInt < iconScale.max && iconScale.progress >= maxScaleInt) {
            hint.setText(R.string.icon_layout_further_increase)
            hint.visibility = View.VISIBLE
        } else {
            hint.visibility = View.GONE
        }

        for (i in 0 until previewIcons.childCount) {
            val tv = previewIcons.getChildAt(i) as BubbleTextView
            tv.applyCellSpecs(cellSpecs)
        }

        if (type == CellLayoutType.DOCK && Pref.Value.dock_background.enabled) {
            updateLayoutForDockBackground()
        }

    }

    public override fun onPause() {
        super.onPause()

        overridePendingTransition(0, 0)

//        prefs.edit()
//            .putInt(Pref.Key.getIconSpecKey(prefix, Pref.Key.ICONSPEC_ICON_SIZE), iconScale.progress)
//            .putBoolean(Pref.Key.getIconSpecKey(prefix, Pref.Key.ICONSPEC_SHOW_LABELS), labelVisible.isChecked)
//            .putString(Pref.Key.getIconSpecKey(prefix, Pref.Key.ICONSPEC_LABEL_FONT_FAMILY), labelFamilyValue)
//            .putInt(Pref.Key.getIconSpecKey(prefix, Pref.Key.ICONSPEC_LABEL_COLOR), labelColor.color)
//            .putBoolean(Pref.Key.getIconSpecKey(prefix, Pref.Key.ICONSPEC_LABEL_SHADOW), labelShadow.isChecked)
//            .putFloat(Pref.Key.getIconSpecKey(prefix, Pref.Key.ICONSPEC_LABEL_SIZE_SP), labelSizeDp)
//            .putBoolean(Pref.Key.getIconSpecKey(prefix, Pref.Key.ICONSPEC_SINGLE_LINE), labelSingleLine.isChecked)
//            .putString(prefix + "_cellspecs", CellIconSizeSpecs.flattenToString(cellSpecs))
//            .apply()
        cellSpecsPref.setIfChanged(cellSpecs.config)

        if (cellSpecsPref == Pref3.desktop_cellspecs || cellSpecsPref == Pref3.drawer_cellspecs) {
            LauncherModel.invalidateCachedIcons(this)
            LauncherAppState.getInstance().model.resetLoadedState(true, true)
        }

        NovaLauncher.postRecreate()
    }
}
