package com.teslacoilsw.launcher.preferences

import android.annotation.TargetApi
import android.app.Activity
import android.app.ActivityOptions
import android.content.Intent
import android.os.Build
import android.view.View
import com.android.launcher3.DragLayer
import com.teslacoilsw.cofs.Cofs
import com.teslacoilsw.cofs.CofsBase
import com.teslacoilsw.cofs.ConfigObjectFromString
import com.teslacoilsw.cofs.getCofsDelegate
import com.teslacoilsw.launcher.R
import com.teslacoilsw.launcher.util.DpUtil


enum class PopupMenuStyle {
    NOUGAT,
    OREO;

    //this == NOUGAT || Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP;
    val isSupported: Boolean
        get() = true

    companion object {
        val default: PopupMenuStyle
            get() = if (OREO.isSupported) OREO else NOUGAT
    }
}

enum class ActivityAnimation constructor(val windowAnimationStyle: Int, internal val mCloseEnter: Int, internal val mCloseExit: Int, internal val mOpenEnter: Int, internal val mOpenExit: Int) {
    SYSTEM(android.R.style.Animation_Activity, 0, 0, 0, 0) {
        override fun overrideCloseTransition(activity: Activity) {}
        override fun makeCustomAnimation(activity: Activity, v: View?, intent: Intent?): ActivityOptions {
            /* Lollipop's system animation has a long delay.
               There isn't a good solution to this for custom roms that will replace it...
             */
            return if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M)
                ActivityOptions.makeClipRevealAnimation(v, 0, 0, v!!.measuredWidth, v.measuredHeight)
            else if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP)
                SLIDE_UP.makeCustomAnimation(activity, v, intent)
            else
                JELLYBEAN.makeCustomAnimation(activity, v, intent)
        }
    },
    ICS(R.style.ActivityAnimation_ICS, R.anim.ics_close_enter, R.anim.ics_close_exit, R.anim.ics_open_enter, R.anim.ics_open_exit),
    JELLYBEAN(R.style.ActivityAnimation_JB, R.anim.jb_close_enter, R.anim.jb_close_exit, R.anim.jb_open_enter, R.anim.jb_open_exit) {
        override fun makeCustomAnimation(activity: Activity, v: View?, intent: Intent?): ActivityOptions {
            if (v != null) {
                if (v is DragLayer && intent != null && intent.sourceBounds != null) {
                    val sourceBounds = intent.sourceBounds
                    return ActivityOptions.makeScaleUpAnimation(v,
                            sourceBounds!!.centerX(), sourceBounds.centerY(), 1, 1)
                }
                return ActivityOptions.makeScaleUpAnimation(v, 0, 0, v.measuredWidth, v.measuredHeight)
            }
            return super.makeCustomAnimation(activity, null, intent)
        }
    },
    SLIDE(R.style.ActivityAnimation_SLIDE, R.anim.slide_close_enter, R.anim.slide_close_exit, R.anim.slide_open_enter, R.anim.slide_open_exit),
    SLIDE_UP(R.style.ActivityAnimation_SLIDE_UP, R.anim.slide_up_close_enter, R.anim.slide_up_close_exit, R.anim.slide_up_open_enter, R.anim.slide_up_open_exit),
    CIRCLE(R.style.ActivityAnimation_SLIDE_UP, R.anim.slide_up_close_enter, R.anim.slide_up_close_exit, R.anim.alpha_activity_open_enter, R.anim.alpha_activity_open_exit),
    MARSHMALLOW(R.style.ActivityAnimation_SLIDE_UP, R.anim.slide_up_close_enter, R.anim.slide_up_close_exit, R.anim.slide_up_open_enter, R.anim.slide_up_open_exit) {
        @TargetApi(Build.VERSION_CODES.M)
        override fun makeCustomAnimation(activity: Activity, v: View?, intent: Intent?): ActivityOptions {
            if (v != null && Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                if (v is DragLayer && intent != null && intent.sourceBounds != null) {
                    /* Handle gestures, which don't have a "view". Intent already has the
                               on screen position for other types of launch animations */
                    val sourceBounds = intent.sourceBounds
                    return ActivityOptions.makeClipRevealAnimation(v,
                            sourceBounds!!.centerX(), sourceBounds.centerY(), DpUtil.getPixelSize(48), DpUtil.getPixelSize(48))
                }
                return ActivityOptions.makeClipRevealAnimation(v, 0, 0,
                        v.measuredWidth, v.measuredHeight)
            }
            return super.makeCustomAnimation(activity, null, intent)
        }
    };

    open fun makeCustomAnimation(activity: Activity, v: View?, intent: Intent?): ActivityOptions {
        return ActivityOptions.makeCustomAnimation(activity, mOpenEnter, mOpenExit)
    }

    open fun overrideCloseTransition(activity: Activity) {
        activity.overridePendingTransition(mCloseEnter, mCloseExit)
    }

}

enum class NightMode {
    DEFAULT, ALWAYS, AUTO, AUTO_DEBUG, CUSTOM, NEVER;

    val isAuto: Boolean
        get() = this == AUTO || this == AUTO_DEBUG || this == CUSTOM
}

enum class GoogleNowTransition {
    PAGE,
    OVERLAY
}

@ConfigObjectFromString
data class PopupItemVisibility(val desktop: Boolean, val drawer: Boolean) {
    companion object : Cofs<PopupItemVisibility>, CofsBase<PopupItemVisibility> by PopupItemVisibility::class.getCofsDelegate() {
        @JvmStatic
        override val DEFAULT = PopupItemVisibility(false, false)
    }
}

@ConfigObjectFromString
data class PopupItems(val uninstall: PopupItemVisibility, val playStore: PopupItemVisibility, val relaunch: PopupItemVisibility) {
    companion object : Cofs<PopupItems>, CofsBase<PopupItems> by PopupItems::class.getCofsDelegate() {
        @JvmStatic
        override val DEFAULT = PopupItems(PopupItemVisibility(false, true), PopupItemVisibility.DEFAULT, PopupItemVisibility.DEFAULT)
    }
}
