package com.teslacoilsw.launcher.preferences

import android.Manifest
import android.content.pm.PackageManager
import android.os.Build
import android.os.Bundle
import android.preference.PreferenceManager
import android.view.MenuItem
import android.view.View
import android.widget.ArrayAdapter
import android.widget.Checkable
import androidx.appcompat.app.ActionBar
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.ContextCompat
import butterknife.ButterKnife
import com.android.launcher3.Utilities
import com.teslacoilsw.launcher.NovaLauncher
import com.teslacoilsw.launcher.R
import com.teslacoilsw.launcher.quicksearchbar.SearchBarStyle
import com.teslacoilsw.launcher.util.SearchUtils
import com.teslacoilsw.launcher.weather.WeatherUnits
import com.teslacoilsw.launcher.widget.DumbRadioGrid
import com.teslacoilsw.shared.colorpicker.ColorUtil
import kotlinx.android.synthetic.main.activity_searchbarsettings.*
import kotlinx.android.synthetic.main.quicksearchbar_base.*

class SearchBarSettingsActivity : AppCompatActivity() {
    private lateinit var style: SearchBarStyle
    private lateinit var prefKey: String

    public override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        val intent = intent
        if (intent == null) {
            finish()
            return
        }

        val key = intent.getStringExtra("preferenceKey")
        if (key.isNullOrEmpty()) {
            finish()
            return
        }
        prefKey = key!!
        val prefValue = PreferenceManager.getDefaultSharedPreferences(this).getString(prefKey, "")
        style = SearchBarStyle.fromString(prefValue)

        setContentView(R.layout.activity_searchbarsettings)

        setSupportActionBar(toolbar)
        supportActionBar!!.displayOptions = ActionBar.DISPLAY_HOME_AS_UP or ActionBar.DISPLAY_SHOW_TITLE

        val showBackArrow = intent.getBooleanExtra("SHOW_BACK_ARROW", true)
        if (!showBackArrow) {
            supportActionBar!!.setHomeAsUpIndicator(R.drawable.ic_menu_done)
        }

        ButterKnife.bind(this)

        (findViewById<View>(getCheckboxIdForLogo(style.logoStyle)) as Checkable).isChecked = true

        radiogroup_logo.setOnCheckedChangeListener(object: DumbRadioGrid.OnCheckedChangeListener {
            override fun onCheckedChanged(group: DumbRadioGrid, checkedId: Int) {
                val logoStyle = when (checkedId) {
                    R.id.logo_material_gray -> SearchBarStyle.Logo.M_MONO
                    R.id.logo_material -> SearchBarStyle.Logo.M_COLOR
                    R.id.logo_classic_color -> SearchBarStyle.Logo.CLASSIC_COLOR
                    R.id.logo_classic -> SearchBarStyle.Logo.CLASSIC_MONO
                    R.id.logo_g -> SearchBarStyle.Logo.M_LETTER_COLOR
                    R.id.logo_g_gray -> SearchBarStyle.Logo.M_LETTER_MONO
                    R.id.logo_magnifying_glass -> SearchBarStyle.Logo.MAGNIFYING_GLASS
                    else -> SearchBarStyle.Logo.M_MONO
                }
                style = SearchBarStyle(logoStyle, style.bgStyle, style.bgColor, style.contentFlags)
                apply()
            }
        })

        when (style.bgStyle) {
            SearchBarStyle.BG_PILLRECT -> bg_style_pillrect.isChecked = true
            SearchBarStyle.BG_ROUNDRECT -> bg_style_roundrect.isChecked = true
            SearchBarStyle.BG_WIRE -> bg_style_wire.isChecked = true
            SearchBarStyle.BG_HONEYCOMB -> bg_style_honeycomb.isChecked = true
            SearchBarStyle.BG_PILL -> bg_style_pill.isChecked = true
            SearchBarStyle.BG_DRAWER_LEGACY -> bg_style_drawer.isChecked = true
            SearchBarStyle.BG_NONE -> { /* no-op */ }
        }

        radiogroup_bg_style.setOnCheckedChangeListener { group, checkedId ->
            var bgStyle = SearchBarStyle.BG_ROUNDRECT

            when (checkedId) {
                R.id.bg_style_pillrect -> bgStyle = SearchBarStyle.BG_PILLRECT
                R.id.bg_style_roundrect -> bgStyle = SearchBarStyle.BG_ROUNDRECT
                R.id.bg_style_wire -> bgStyle = SearchBarStyle.BG_WIRE
                R.id.bg_style_honeycomb -> bgStyle = SearchBarStyle.BG_HONEYCOMB
                R.id.bg_style_pill -> bgStyle = SearchBarStyle.BG_PILL
                R.id.bg_style_drawer -> bgStyle = SearchBarStyle.BG_DRAWER_LEGACY
            }

            var bgColor = bg_color.color
            if (!SearchBarStyle.supportsTransparency(bgStyle))
                bgColor = ColorUtil.argb(255, bgColor)

            style = SearchBarStyle(style.logoStyle, bgStyle, bgColor, style.contentFlags)
            apply()
        }

        fun removeRadioBg(view: View) {
            val idx = radiogroup_bg_style.indexOfChild(view)
            radiogroup_bg_style.removeViewAt(idx)
            radiogroup_bg_style_previews.removeViewAt(idx)
        }

        if (prefKey == Pref3.drawer_search_style.key) {
            removeRadioBg(bg_style_pill)
            removeRadioBg(bg_style_honeycomb)
        } else {
            removeRadioBg(bg_style_drawer)
        }

        bg_color.color = style.bgColor
        bg_color.onUserChanged = { _, _ ->
            style = SearchBarStyle(style.logoStyle, style.bgStyle, bg_color.color, style.contentFlags)
            apply()
        }

        val adapter = ArrayAdapter<Any>(this@SearchBarSettingsActivity, R.layout.spinner_textview_14sp)
        adapter.add(getString(R.string.weather_celsius))
        adapter.add(getString(R.string.weather_fahrenheit))
        weather_units_spinner.value = Pref2.get().weather_units.get()!!.name
        weather_units_spinner.onUserChanged = { _, _ ->
                Pref2.get().weather_units.set(WeatherUnits.valueOf(weather_units_spinner.value))
                apply()
        }

        weather.isChecked = style.hasWeather()
        weather.onUserChanged = { _, _ ->
            var doApply = true
            if (weather.isChecked && Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                if (ContextCompat.checkSelfPermission(this@SearchBarSettingsActivity, Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
                    requestPermissions(arrayOf(Manifest.permission.ACCESS_FINE_LOCATION), 1)
                    doApply = false
                }
            }

            if (doApply) {
                style = SearchBarStyle(style.logoStyle, style.bgStyle, style.bgColor, Utilities.alterFlags(style.contentFlags, SearchBarStyle.CONTENT_WEATHER, weather.isChecked))
                apply()
            }
        }

        voice.isChecked = Utilities.hasFlag(style.contentFlags, SearchBarStyle.CONTENT_VOICE)
        voice.onUserChanged = { _, _ ->
            style = SearchBarStyle(style.logoStyle, style.bgStyle, style.bgColor, Utilities.alterFlags(style.contentFlags, SearchBarStyle.CONTENT_VOICE, voice.isChecked))
            apply()
        }

        if (prefKey == "qsb_style") {
            if (SearchUtils.getSearchOverlayActivity(this) == null) {
                val entries = search_window_style.entries
                entries.toMutableList().removeAll { it.value == SearchBarStyle.SearchWindowStyle.GOOGLE_OVERLAY.name }
                search_window_style.entries = entries
            }
            search_window_style.value = Pref3.search_window_style.value.name
        } else {
            search_window_style.visibility = View.GONE
        }

        if (!style.hasWeather() && !Pref.Value.mPrefs.getBoolean("labs_show_weather", false)) {
            weather_units_spinner.visibility = View.GONE
            weather.visibility = View.GONE
        }

        sneeze_guard.setOnTouchListener{ _, _ -> true }
        sneeze_guard.setOnClickListener {  }

        apply()
    }

    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<String>, grantResults: IntArray) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)

        weather.isChecked = ContextCompat.checkSelfPermission(this@SearchBarSettingsActivity, android.Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_GRANTED
        style = SearchBarStyle(style.logoStyle, style.bgStyle, style.bgColor, Utilities.alterFlags(style.contentFlags, SearchBarStyle.CONTENT_WEATHER, weather.isChecked))
        apply()
    }

    private fun apply() {

        if (bg_style_drawer.isChecked) {
            qsb_base_background.visibility = View.INVISIBLE
            preview_drawer.visibility = View.VISIBLE
        } else {
            qsb_base_background.visibility = View.VISIBLE
            preview_drawer.visibility = View.GONE
            style.apply(qsb_base_background)
        }

        voice.isEnabled = style.bgStyle != SearchBarStyle.BG_PILL
        weather_units_spinner.isEnabled = weather.isChecked

        val visibility = if (bg_style_drawer.isChecked) View.GONE else View.VISIBLE
        logo_header.visibility = visibility
        radiogroup_logo.visibility = visibility
        content_header.visibility = visibility
        voice.visibility = visibility
        divider.visibility = visibility
        bg_color.isEnabled = visibility != View.GONE
    }


    private fun getCheckboxIdForLogo(logo: SearchBarStyle.Logo): Int {
        return when (logo) {
            SearchBarStyle.Logo.M_MONO -> R.id.logo_material_gray
            SearchBarStyle.Logo.M_COLOR -> R.id.logo_material
            SearchBarStyle.Logo.CLASSIC_COLOR -> R.id.logo_classic_color
            SearchBarStyle.Logo.CLASSIC_MONO -> R.id.logo_classic
            SearchBarStyle.Logo.M_LETTER_COLOR -> R.id.logo_g
            SearchBarStyle.Logo.M_LETTER_MONO -> R.id.logo_g_gray
            SearchBarStyle.Logo.MAGNIFYING_GLASS -> R.id.logo_magnifying_glass
            else -> throw IllegalArgumentException("Unknown logo type $logo")
        }
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when (item.itemId) {
            android.R.id.home -> {
                onBackPressed()
                return true
            }
        }

        return super.onOptionsItemSelected(item)
    }

    private fun clearTouchFeedback() {
        qsb_base_touchfeedback.clearTouchFeedback()
    }

    fun onClickVoiceButton(v: View) {
        clearTouchFeedback()
    }

    fun onClickAllAppsButton(v: View) {
        clearTouchFeedback()
    }

    fun onClickDate(v: View) {
        clearTouchFeedback()
    }

    override fun onPause() {
        super.onPause()

        PreferenceManager.getDefaultSharedPreferences(this).edit().putString(prefKey, style.flattenToString()).apply()
        if (search_window_style.visibility == View.VISIBLE)
            Pref3.search_window_style.setIfChanged(SearchBarStyle.SearchWindowStyle.valueOf(search_window_style.value))
        if (prefKey == Pref3.drawer_search_style.key)
            NovaLauncher.postRecreate()
    }
}
