package com.teslacoilsw.launcher.preferences.fancyprefs

import android.annotation.SuppressLint
import android.content.Context
import android.graphics.drawable.Drawable
import android.util.AttributeSet
import android.widget.Checkable
import android.widget.CompoundButton
import android.widget.FrameLayout
import androidx.core.view.get
import androidx.core.view.updateLayoutParams
import com.teslacoilsw.launcher.R
import com.teslacoilsw.launcher.preferences.widget.UserSwitchCompat

class FancyPrefCheckableView @JvmOverloads
    constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0)
    : FancyPrefView<Boolean>(context, attrs, defStyleAttr), Checkable
{
    private val checkable: CompoundButton
    override fun isChecked(): Boolean = value
    override fun toggle() { value = !value }
    override fun setChecked(checked: Boolean) { value = checked }
    var checkMarkDrawable: Drawable?
        @SuppressLint("NewApi")
        get() = checkable.buttonDrawable
        set(value) {
            checkable.buttonDrawable = value
        }


    init {
        val layoutResource: Int
        val a = context.obtainStyledAttributes(attrs, R.styleable.FancyPrefCheckableView)
        val widgetFrame = findViewById<FrameLayout>(R.id.widget_frame)
        try {
            layoutResource = if (a.getBoolean(R.styleable.FancyPrefCheckableView_useSwitch, false)) {
                widgetFrame.updateLayoutParams<MarginLayoutParams> {
                    this.marginEnd = 0
                }
                R.layout.fancypref_widget_switch
           } else {
                R.layout.fancypref_widget_checkbox
            }
            setWidget(layoutResource)
            checkable = widgetFrame[0] as CompoundButton
            value = a.getBoolean(R.styleable.FancyPrefCheckableView_android_checked, false)
        } finally {
            a.recycle()
        }

        if (checkable is UserSwitchCompat) {
            checkable.setOnCheckedChangeListener { _, _ ->
                if (checkable.gettingUserInput)
                    setFromUser(checkable.isChecked)
            }
        }
        setOnClickListener {
            setFromUser(!value)
        }
    }

    override fun onChange(value: Boolean) {
        super.onChange(value)
        checkable.isChecked = value
    }
}