package com.teslacoilsw.launcher.preferences.fancyprefs

import android.content.Context
import android.graphics.Color
import android.util.AttributeSet
import androidx.annotation.ColorInt
import androidx.appcompat.app.AppCompatActivity
import androidx.core.view.get
import com.teslacoilsw.kotlin.getActivity
import com.teslacoilsw.launcher.R
import com.teslacoilsw.launcher.preferences.COLOR_AUTO
import com.teslacoilsw.launcher.preferences.fancyprefs.color.ColorPickerBottomSheet
import com.teslacoilsw.launcher.preferences.fancyprefs.color.ColorPickerButton
import com.teslacoilsw.shared.colorpicker.ColorUtil
import kotlinx.android.synthetic.main.fancypref_row.view.*

class FancyPrefColorView @JvmOverloads
    constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0)
    : FancyPrefView<Int>(context, attrs, defStyleAttr)
{
    private val canAlpha: Boolean
    private val canManage: Boolean

    var defaultColor: Int
    private val colorWidget: ColorPickerButton

    var color: Int
        @ColorInt get() = colorWidget.color
        set(@ColorInt newColor) {
            if (canAlpha || (canManage && newColor == COLOR_AUTO))
                colorWidget.color = newColor
            else
                colorWidget.color = ColorUtil.argb(0xFF, newColor)
            value = colorWidget.color
        }

    var dialogTitle: String? = null

    init {
        setWidget(R.layout.fancypref_widget_color)
        colorWidget = widget_frame[0] as ColorPickerButton
        val a = context.obtainStyledAttributes(attrs, R.styleable.FancyPrefColorView)
        try {
            dialogTitle = a.getString(R.styleable.FancyPrefColorView_android_dialogTitle)?: titleView?.text.toString()
            canAlpha = a.getBoolean(R.styleable.FancyPrefColorView_canAlpha, false)
            canManage = a.getBoolean(R.styleable.FancyPrefColorView_canManagedByNova, false)
            color = a.getInt(R.styleable.FancyPrefColorView_android_color, Color.WHITE)
            defaultColor = a.getInt(R.styleable.FancyPrefColorView_defaultColor, Color.TRANSPARENT)
        } finally {
            a.recycle()
        }

        setOnClickListener {
            val dialog = ColorPickerBottomSheet.newInstance(dialogTitle, color, canAlpha, canManage, defaultColor)
            dialog.setOnColorSelectedListener { result ->
                color = result
                setFromUser(result)
            }
            dialog.show(context.getActivity() as AppCompatActivity)
        }
    }

    override fun onChange(value: Int) {
        super.onChange(value)
        color = value
    }
}