package com.teslacoilsw.launcher.preferences.fancyprefs

import android.content.Context
import android.content.DialogInterface
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import android.widget.FrameLayout
import com.afollestad.materialdialogs.DialogAction
import com.afollestad.materialdialogs.MaterialDialog
import com.teslacoilsw.launcher.R

abstract class FancyPrefDialogView<T : Any> @JvmOverloads
constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0)
    : FancyPrefView<T>(context, attrs, defStyleAttr) {

    protected var dialog: DialogInterface? = null
    var dialogLayoutId: Int = 0
    var dialogTitle: CharSequence? = null
    var negativeButtonLabel: String? = context.getString(R.string.cancel)
    var positiveButtonLabel: String? = context.getString(R.string.done)
    /** This is not meant for subclasses but rather the consumer */
    var onBindDialog: ((dialogContent: View) -> Unit)? = null
    var onPositive: ((dialogContent: View) -> Unit)? = null

    init {
        val a = context.obtainStyledAttributes(attrs, R.styleable.FancyPrefDialogView)
        try {
            dialogTitle = a.getString(R.styleable.FancyPrefDialogView_android_dialogTitle) ?: titleView?.text
            dialogLayoutId = a.getResourceId(R.styleable.FancyPrefDialogView_dialogLayout, 0)
        } finally {
            a.recycle()
        }

        setOnClickListener {
            showDialog()
        }

    }

    /**
     * Returns lambda to compute a return value to pass onto onDismiss
     */
    abstract fun bindDialogView(dialogView: View) : (() -> T)?

    protected open fun showDialog(): DialogInterface {
        val builder = MaterialDialog.Builder(context)
        positiveButtonLabel?.let {
            negativeButtonLabel?.let { builder.negativeText(it) }
            builder.positiveText(it)
        }
        dialogTitle?.let { builder.title(it) }

        val v = if (dialogLayoutId != 0)
            LayoutInflater.from(context).inflate(dialogLayoutId, null, false)
        else
            FrameLayout(context)
        val resultCallback = bindDialogView(v)
        onBindDialog?.invoke(v)

        builder.customView(v, false)
        val defaultAction = if (positiveButtonLabel == null)
            DialogAction.POSITIVE else DialogAction.NEGATIVE
        val buttonSelected = arrayOf(defaultAction)
        builder.onAny { _, which ->
            buttonSelected[0] = which
        }
        val dialog = builder.show()
        dialog.setOnDismissListener {
            val result = resultCallback?.invoke()
            if (buttonSelected[0] == DialogAction.POSITIVE && result != null) {
                setFromUser(result)
                onPositive?.invoke(v)
            }
            this.dialog = null
        }
        this.dialog = dialog
        return dialog
    }

    fun dismiss() {
        dialog?.dismiss()
    }

}