package com.teslacoilsw.launcher.preferences.fancyprefs

import android.content.Context
import android.util.AttributeSet
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.ScrollView
import androidx.core.view.updateLayoutParams
import com.teslacoilsw.kotlin.dp
import com.teslacoilsw.launcher.R
import com.teslacoilsw.launcher.preferences.widget.MatchWrapLinearLayout

class FancyPrefExpanderView @JvmOverloads
constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0)
    : MatchWrapLinearLayout(context, attrs, defStyleAttr)
{
    private val imageView: ImageView
    var isExpanded = true
        set(value) {
            field = value
            imageView.setImageResource(if (value) R.drawable.ic_action_arrow_up else R.drawable.ic_action_arrow_down)
            requestLayout()
            if (value) {
                handler?.post {
                    val scrollView = parent?.parent as? ScrollView
                    if (scrollView != null) {
                        val viewportBottom = scrollView.scrollY + scrollView.height
                        if (viewportBottom < top + height) {
                            val viewAtTopPos = top
                            val viewBottomAtBottomPos = top + height - scrollView.height
                            if (viewBottomAtBottomPos - height > viewAtTopPos) {
                                scrollView.smoothScrollTo(0, viewBottomAtBottomPos)
                            } else {
                                scrollView.smoothScrollTo(0, viewAtTopPos)
                            }
                        }
                    }
                }
            }
        }

    init {
        orientation = LinearLayout.VERTICAL

        val fancypref = FancyPrefIconView(context, attrs)
        fancypref.widgetFrame!!.updateLayoutParams<MarginLayoutParams> {
            marginEnd = context.dp(8)
        }
        imageView = fancypref.findViewById(R.id.widgetIcon)
        val lp = LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.WRAP_CONTENT)
        addView(fancypref, 0, lp)

        fancypref.setOnClickListener {
            isExpanded = !isExpanded
        }
        isExpanded = false
    }

    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        if (isExpanded)
            super.onMeasure(widthMeasureSpec, heightMeasureSpec)
        else {
            getChildAt(0).measure(MeasureSpec.makeMeasureSpec(MeasureSpec.getSize(widthMeasureSpec), MeasureSpec.EXACTLY), MeasureSpec.makeMeasureSpec(MeasureSpec.getSize(heightMeasureSpec), MeasureSpec.UNSPECIFIED))
            setMeasuredDimension(getChildAt(0).measuredWidth, getChildAt(0).measuredHeight)
        }
    }

    override fun onLayout(changed: Boolean, l: Int, t: Int, b: Int, r: Int) {
        if (isExpanded) {
            super.onLayout(changed, l, t, r, b)
        } else {
            getChildAt(0).layout(0, 0, measuredWidth, measuredHeight)
        }
    }
}