package com.teslacoilsw.launcher.preferences.fancyprefs

import android.content.Context
import android.util.AttributeSet
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.core.view.isVisible
import androidx.core.view.updateLayoutParams
import com.teslacoilsw.kotlin.dp
import com.teslacoilsw.launcher.R
import com.teslacoilsw.launcher.preferences.Pref3
import com.teslacoilsw.launcher.widget.GridPreviewView
import com.teslacoilsw.launcher.widget.NumberPicker
import com.teslacoilsw.shared.util.MathUtils

open class FancyPrefGridView @JvmOverloads
    constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0)
    : FancyPrefDialogView<Pair<Int, Int>>(context, attrs, defStyleAttr)
{
    private val gridPreview: GridPreviewView
    private val autoText: TextView

    var minRows: Int
    var maxRows: Int
    var minCols: Int
    var maxCols: Int
    var disabledSummary: String?
    var subgridPref: Pref3.Pref<Boolean>? = null

    init {
        if (dialogLayoutId == 0)
            dialogLayoutId = R.layout.dialog_preference_grid
        if (summary == null)
            summary = "%d ✕ %d"

        setWidget(R.layout.fancypref_widget_grid)

        val a = context.obtainStyledAttributes(attrs, R.styleable.FancyPrefGridView)
        try {
            minRows = a.getInt(R.styleable.FancyPrefGridView_minRows, 2)
            maxRows = a.getInt(R.styleable.FancyPrefGridView_maxRows, 8)
            minCols = a.getInt(R.styleable.FancyPrefGridView_minCols, 2)
            maxCols = a.getInt(R.styleable.FancyPrefGridView_maxCols, 8)
            disabledSummary = a.getString(R.styleable.FancyPrefGridView_disabledSummary)
        } finally {
            a.recycle()
        }
        gridPreview = widgetFrame!!.findViewById(R.id.gridPreview)
        autoText = widgetFrame.findViewById(R.id.auto)

        summaryView?.textSize = 12f
        value = 3 to 4

        gridPreview.updateLayoutParams<ViewGroup.LayoutParams> {
            val aspectRatio = 0.8f //getAspectRatio()
            val size = context.dp(48)
            if (aspectRatio > 1) {
                height = MathUtils.roundInt(size * 1 / aspectRatio)
                width = size
            } else {
                width = MathUtils.roundInt(size * aspectRatio)
                height = size
            }
        }
        updateSummary()
    }

    override fun setEnabled(enabled: Boolean) {
        super.setEnabled(enabled)
        if (enabled) {
            autoText.visibility = View.INVISIBLE
            gridPreview.visibility = View.VISIBLE
        } else {
            autoText.visibility = View.VISIBLE
            gridPreview.visibility = View.INVISIBLE
        }
        updateSummary()
    }

    override fun bindDialogView(v: View): (() -> Pair<Int, Int>)? {
        val colPicker = v.findViewById(R.id.cols) as NumberPicker
        val rowPicker = v.findViewById(R.id.rows) as NumberPicker

        if (subgridPref != null) {
            val subgrid: FancyPrefCheckableView = v.findViewById(R.id.subgrid)
            subgrid.isVisible = true
            subgrid.bind(subgridPref)
        }

        val gridPreview = v.findViewById(R.id.preview) as GridPreviewView
        gridPreview.setGridSize(this.gridPreview.rows, this.gridPreview.cols)

//        val seekBarListener: ((Int, Int) -> Unit) = { _, _ ->
//            gridPreview.setGridSize(rowPicker.progress, colPicker.progress)
//        }
        val pickerListener = NumberPicker.OnValueChangeListener { picker, old, new ->
            gridPreview.setGridSize(rowPicker.value, colPicker.value)
        }

        colPicker.maxValue = maxCols
        colPicker.minValue = minCols
        colPicker.value = gridPreview.cols
        colPicker.setOnValueChangedListener(pickerListener)
//        colPicker.onUserChanged = seekBarListener

        rowPicker.maxValue = maxRows
        rowPicker.minValue = minRows
        rowPicker.value = gridPreview.rows
        rowPicker.setOnValueChangedListener(pickerListener)
//        rowPicker.onUserChanged = seekBarListener

        return { rowPicker.value to colPicker.value }
    }

    override fun onChange(value: Pair<Int, Int>) {
        super.onChange(value)
        gridPreview.setGridSize(value.first, value.second)
        updateSummary()
    }

    override fun updateSummary() {
        val s = if (isEnabled) {
            summary
        } else {
            when (disabledSummary) {
                null -> summary
                "" -> null
                else -> disabledSummary
            }
        }
        val text = if (s != null && s.contains('%') && valueFieldInitialized) {
            summaryFormat(s.toString(), value)
        } else {
            s
        }
        updateSummaryViews(text)
    }

     override fun summaryFormat(text: String, value: Pair<Int, Int>): String =
         String.format(text, value.first, value.second)
}