package com.teslacoilsw.launcher.preferences.fancyprefs

import android.content.Context
import android.graphics.drawable.Drawable
import android.util.AttributeSet
import android.util.TypedValue
import android.widget.ImageView
import com.teslacoilsw.kotlin.lazy1
import com.teslacoilsw.launcher.R
import kotlin.math.roundToInt

class FancyPrefIconView @JvmOverloads
    constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0)
    : FancyPrefView<Unit>(context, attrs, defStyleAttr)
{
    private val disabledAlpha by lazy1 {
        val tv = TypedValue()
        resources.getValue(R.dimen.disabled_alpha_material_light, tv, true)
        (255f * tv.float).roundToInt()
    }
    private val widgetIconView: ImageView

    var widgetIcon: Drawable?
        get() = widgetIconView.drawable
        set(value) { widgetIconView.setImageDrawable(value) }

    fun setWidgetIconRes(resourceId: Int) {
        widgetIconView.setImageResource(resourceId)
    }

    init {
        setWidget(R.layout.fancypref_widget_icon)
        widgetIconView = findViewById(R.id.widgetIcon)

        val a = context.obtainStyledAttributes(attrs, R.styleable.FancyPrefIconView)
        try {
            a.getDrawable(R.styleable.FancyPrefIconView_android_src)?.let {
                widgetIconView.setImageDrawable(it)
            }
            val iconSize = a.getDimensionPixelSize(R.styleable.FancyPrefIconView_iconSize, 0)
            if (iconSize != 0) {
                val lp = widgetIconView.layoutParams
                lp.width = iconSize
                lp.height = iconSize
            }
            if (a.hasValue(R.styleable.FancyPrefIconView_widgetIconTint)) {
                widgetIconView.imageTintList = a.getColorStateList(R.styleable.FancyPrefIconView_widgetIconTint)
            }
        } finally {
            a.recycle()
        }
    }

    override fun setEnabled(enabled: Boolean) {
        super.setEnabled(enabled)
        widgetIconView.imageAlpha = if (enabled) 255 else disabledAlpha
    }
}