package com.teslacoilsw.launcher.preferences.fancyprefs

import android.content.Context
import android.util.AttributeSet
import android.view.View
import android.view.ViewGroup
import android.widget.AdapterView
import android.widget.ArrayAdapter
import android.widget.Spinner
import androidx.core.view.updateLayoutParams
import com.android.launcher3.LauncherAppState
import com.teslacoilsw.launcher.R
import com.teslacoilsw.launcher.widget.GridPreviewView
import com.teslacoilsw.launcher.widget.NumberPicker
import kotlinx.android.synthetic.main.dialog_preference_grid.view.*
import java.util.concurrent.atomic.AtomicBoolean

class FancyPrefLandGridView @JvmOverloads
constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0)
    : FancyPrefGridView(context, attrs, defStyleAttr) {

    lateinit var landValue: Pair<Int, Int>

    override fun bindDialogView(v: View): (() -> Pair<Int, Int>)? {
        super.bindDialogView(v)

        val colPicker = v.findViewById(R.id.cols) as NumberPicker
        val rowPicker = v.findViewById(R.id.rows) as NumberPicker

        rowPicker.value = value.first
        colPicker.value = value.second
        val gridPreview = v.findViewById(R.id.preview) as GridPreviewView
        gridPreview.setGridSize(rowPicker.value, colPicker.value)

        val dialogValue = arrayOf(value, landValue)

        val isPortrait = AtomicBoolean(true)

        val orientationSpinner = v.findViewById<Spinner>(R.id.orientationSpinner)
        orientationSpinner.visibility = View.VISIBLE
        if (orientationSpinner.adapter == null) {
            val orientations = arrayOf(v.resources.getString(R.string.portrait), v.resources.getString(R.string.landscape))
            val adapter = ArrayAdapter<CharSequence>(context,
                    android.R.layout.simple_spinner_item, orientations)
            adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
            orientationSpinner.adapter = adapter
            orientationSpinner.setSelection(if (LauncherAppState.isScreenLandscape(context)) 1 else 0)

            orientationSpinner.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {
                override fun onItemSelected(parent: AdapterView<*>, view: View, position: Int, id: Long) {
                    val w = context.resources.getDimensionPixelSize( if (position == 0)
                        R.dimen.fancypref_padding_preview_width
                    else
                        R.dimen.fancypref_padding_preview_land_width)

                    val h = context.resources.getDimensionPixelSize( if (position == 0)
                        R.dimen.fancypref_padding_preview_height
                    else
                        R.dimen.fancypref_padding_preview_land_height)

                    v.preview_bg.updateLayoutParams<ViewGroup.LayoutParams> { width = w; height = h  }
                    v.preview.updateLayoutParams<ViewGroup.LayoutParams> { width = w; height = h  }

                    if (isPortrait.get())
                        dialogValue[0] = v.rows.value to v.cols.value
                    else
                        dialogValue[1] = v.rows.value to v.cols.value

                    isPortrait.set(position == 0)
                    if (isPortrait.get()) {
                        v.rows.value = dialogValue[0].first
                        v.cols.value = dialogValue[0].second
                    } else {
                        v.rows.value = dialogValue[1].first
                        v.cols.value = dialogValue[1].second
                    }

                    v.preview.setGridSize(v.rows.value, v.cols.value)
                }

                override fun onNothingSelected(parent: AdapterView<*>) {

                }
            }

        }

        return {
            if (isPortrait.get())
                dialogValue[0] = v.rows.value to v.cols.value
            else
                dialogValue[1] = v.rows.value to v.cols.value
            landValue = dialogValue[1]
            dialogValue[0]
        }
    }
}
