package com.teslacoilsw.launcher.preferences.fancyprefs

import android.content.Context
import android.content.res.ColorStateList
import android.util.AttributeSet
import android.view.View
import android.view.ViewGroup
import com.teslacoilsw.kotlin.getThemeAttrColor
import com.teslacoilsw.kotlin.getThemeAttrDrawable

class FancyPrefSummaryListMultiView @JvmOverloads
constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0)
    : FancyPrefSummaryListBaseView<Set<String>>(context, attrs, defStyleAttr)
{
    private lateinit var dialogValue: MutableSet<String>

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int, clickListener: OnClickListener): SummaryListViewHolder {
        val holder = super.onCreateViewHolder(parent, viewType, clickListener)
        holder.fancyPref.checkMarkDrawable = context.getThemeAttrDrawable(android.R.attr.listChoiceIndicatorMultiple)
        holder.icon.imageTintList = ColorStateList.valueOf(context.getThemeAttrColor(android.R.attr.textColorSecondary))
        return holder
    }

    override fun bindDialogView(dialogView: View): (() -> Set<String>)? {
        dialogValue = value.toMutableSet()
        super.bindDialogView(dialogView)

        return { dialogValue }
    }

    override fun onSelectedInDialog(dialogView: View, entry: SummaryListRowEntry) {
        if (dialogValue.contains(entry.value))
            dialogValue.remove(entry.value)
        else
            dialogValue.add(entry.value)
    }

    override fun isChecked(value: String) =
            dialogValue.contains(value)

    override fun summaryFormat(text: String, value: Set<String>): String {
        val sortedSetTitles = entries.filter { value.contains(it.value) }.map { it.title }
        return String.format(text, sortedSetTitles.joinToString(", "))
    }

    override fun requireConfirmation() = true

}