package com.teslacoilsw.launcher.preferences.fancyprefs

import android.content.Context
import android.util.AttributeSet
import android.view.View
import android.widget.ImageView
import android.widget.TextView
import com.teslacoilsw.launcher.R

private const val RESULT_DISPLAY_NONE = 0
private const val RESULT_DISPLAY_SUMMARY = 1
private const val RESULT_DISPLAY_IMAGE = 2


open class FancyPrefSummaryListView @JvmOverloads
constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0)
    : FancyPrefSummaryListBaseView<String>(context, attrs, defStyleAttr)
{
    val resultDisplay: Int
    private val resultImage: ImageView?
    private val resultText: TextView?

    private lateinit var dialogValue: String

    init {
        val a = context.obtainStyledAttributes(attrs, R.styleable.FancyPrefSummaryListView)
        try {
            resultDisplay = a.getInt(R.styleable.FancyPrefSummaryListView_resultDisplay, 0)
        } finally {
            a.recycle()
        }

        if (resultDisplay == RESULT_DISPLAY_IMAGE)
            setWidget(R.layout.fancypref_widget_iconortext)

        resultImage = findViewById(R.id.widgetIcon)
        resultText = findViewById(R.id.widgetText)
    }

    override fun bindDialogView(dialogView: View): (() -> String)? {
        dialogValue = value
        super.bindDialogView(dialogView)

        return { dialogValue }
    }

    override fun onSelectedInDialog(dialogView: View, entry: SummaryListRowEntry) {
        dialogValue = entry.value
    }

    override fun isChecked(value: String) =
        dialogValue == value

    override fun onChange(value: String) {
        super.onChange(value)
        if (resultDisplay == RESULT_DISPLAY_IMAGE) {
            val entry = entryForValue(value ?: "")
            resultImage!!.setImage(entry)
            resultText!!.text = if (entry?.hasIcon == true) null else entry?.title
        }
    }

    override fun summaryFormat(text: String, value: String): String {
        return String.format(text, entryForValue(value)?.title)
    }
}

private fun ImageView.setImage(entry: SummaryListRowEntry?) {
    visibility = when {
        entry == null -> {
            setImageDrawable(null)
            View.GONE
        }
        entry.iconDrawable != null -> {
            setImageDrawable(entry.iconDrawable)
            View.VISIBLE
        }
        entry.iconResId != 0 -> {
            setImageResource(entry.iconResId)
            View.VISIBLE
        }
        else -> {
            setImageDrawable(null)
            View.GONE
        }
    }
}