package com.teslacoilsw.launcher.preferences.fragments

import android.content.SharedPreferences
import android.os.Bundle
import android.preference.PreferenceManager
import android.view.View
import android.view.ViewGroup
import android.view.ViewParent
import android.widget.ScrollView
import androidx.core.view.postDelayed
import com.teslacoilsw.kotlin.locationInParent
import com.teslacoilsw.kotlin.size
import com.teslacoilsw.launcher.R
import com.teslacoilsw.launcher.preferences.fancyprefs.FancyPrefExpanderView
import com.teslacoilsw.launcher.preferences.fancyprefs.FancyPrefView
import timber.log.Timber

open class NovaSettingsFragment : NovaPreferenceFragment() {
    lateinit var sharedPrefs: SharedPreferences
        private set
    private var firstResume: Boolean = true
    protected var targetPreference: Int? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        sharedPrefs = PreferenceManager.getDefaultSharedPreferences(activity)
        targetPreference = arguments?.getBundle("intentArgs")?.getInt("targetPref")
        if (targetPreference != null)
            Timber.i("NovaSettingsFragment onCreate $targetPreference")
    }


    override fun onResume() {
        super.onResume()
        if (firstResume) {
            if (targetPreference != null) {
                val scrollView = view.findViewById<ScrollView>(R.id.scrollview)
                val target = view.findViewById<View>(targetPreference!!)
                if (target.visibility == View.GONE)
                    target.visibility = View.VISIBLE
                if (target != null) {
                    var parent: ViewParent? = target.parent
                    while (parent != null && parent is ViewGroup) {
                        if (parent is FancyPrefExpanderView) {
                            parent.isExpanded = true
                        }
                        parent = parent.parent
                    }
                    target.post {
                        val y = target.locationInParent(scrollView).y.toInt() - scrollView.measuredHeight/2
                        scrollView.scrollTo(0, y)
                    }
                    target.postDelayed(700) {
                        target.background?.setHotspot(target.width/2f, target.height/2f)
                        target.isPressed = true
                        target.isPressed = false
                        target.postDelayed(300) {
                            target.background?.setHotspot(target.width/2f, target.height/2f)
                            target.isPressed = true
                            target.isPressed = false
                        }
                    }
                }

            }

            firstResume = false
        }

        refreshPrimeState(view)
    }

    private fun refreshPrimeState(view: View) {
        if (view is FancyPrefView<*>) {
            view.refreshPrimeState()
        } else if (view is ViewGroup) {
            for (i in 0 until view.size) {
                refreshPrimeState(view.getChildAt(i))
            }
        }
    }



    /* Method refernece for use with onUserChange */
    fun requireRestart(old: Any, new: Any) {
        requireRestart()
    }
}