package com.teslacoilsw.launcher.preferences.fragments

import android.annotation.SuppressLint
import android.app.FragmentTransaction
import android.content.Intent
import android.content.pm.PackageManager
import android.net.Uri
import android.os.Build
import android.os.Bundle
import android.provider.Settings
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.core.content.ContextCompat
import com.teslacoilsw.cofs.internal.parse
import com.teslacoilsw.kotlin.bind
import com.teslacoilsw.launcher.BuildConfig
import com.teslacoilsw.launcher.NewAppShortcutReceiver
import com.teslacoilsw.launcher.R
import com.teslacoilsw.launcher.databinding.SettingsDesktopBinding
import com.teslacoilsw.launcher.preferences.IconLayoutSettingsActivity
import com.teslacoilsw.launcher.preferences.Pref
import com.teslacoilsw.launcher.preferences.Pref2
import com.teslacoilsw.launcher.preferences.Pref3
import com.teslacoilsw.launcher.preferences.SearchBarPlacement
import com.teslacoilsw.launcher.preferences.SearchBarSettingsActivity
import com.teslacoilsw.launcher.preferences.WallpaperScrollMode
import com.teslacoilsw.launcher.preferences.widget.ScrollEffectPreference
import com.teslacoilsw.launcher.util.PackageNames

private const val REQUEST_PERMISSIONS_FOR_FORCE_WALLPAPER = 43840702

class SettingsDesktop: NovaSettingsFragment() {
    private lateinit var binding : SettingsDesktopBinding

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup, savedInstanceState: Bundle?): View {
        binding = inflater.bind(R.layout.settings_desktop, container, false)

        binding.iconLayout.setOnClickListener {
            val intent = Intent(activity, IconLayoutSettingsActivity::class.java)
            intent.putExtra("category", "DESKTOP")
            startActivity(intent)
        }

        binding.dock.setOnClickListener {
            val dock = SettingsDock()
            (activity).fragmentManager.beginTransaction()
                    .replace(R.id.fragment_container, dock, "SETTINGS_FRAGMENT")
                    .setTransition(FragmentTransaction.TRANSIT_FRAGMENT_OPEN)
                    .addToBackStack(null)
                    .commit()
        }

        if (sharedPrefs.getBoolean("big_grid_size", false)) {
            binding.grid.maxCols = 16
            binding.grid.maxRows = 16
        }
        binding.grid.subgridPref = Pref3.desktop_subgrid
        binding.grid.onUserChanged = { _, it ->
            sharedPrefs.edit()
                    .putInt(Pref.Key.DESKTOP_GRID_ROWS, it.first)
                    .putInt(Pref.Key.DESKTOP_GRID_COLS, it.second)
                    .apply()
            requireRestart()
        }
        binding.padding.onUserChanged = { _, it ->
            sharedPrefs.edit()
                    .putString(Pref.Key.DESKTOP_WIDTH_MARGIN, it.first.name)
                    .putString(Pref.Key.DESKTOP_HEIGHT_MARGIN, it.second.name)
                    .apply()
            requireRestart()
        }

        if (!Pref.Value.nova_prime_enabled)
            binding.scrollEffect.entries = binding.scrollEffect.entries.subList(0, 3)
        fun anim(workspace: ScrollEffectPreference.WorkspaceView) {
            var newPage = workspace.nextPage + 1
            if (newPage >= 2)
                newPage = 0
            workspace.snapToPage(newPage, 1500)
        }
        binding.scrollEffect.onBindDialog = { dialogContent ->
            dialogContent.findViewById<ScrollEffectPreference.WorkspaceView>(R.id.workspace).setOnClickListener {
                anim(it as ScrollEffectPreference.WorkspaceView)
            }
        }
        binding.scrollEffect.onSelectedInDialog = { dialogContent, entry ->
            val workspace = dialogContent.findViewById<ScrollEffectPreference.WorkspaceView>(R.id.workspace)
            workspace.setScrollEffect(Pref.Key.ScrollEffectEnum.valueOf(entry.value))
            anim(workspace)
        }

        binding.wallpaperScrolling.onUserChanged = { _, new ->
            if (WallpaperScrollMode.DISABLED.name == new)
                Pref3.wallpaper_scrolling_force.setIfChanged(false)
        }
        binding.wallpaperScrollingForce.onUserChanged = { old, new ->
            if (new) {
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O && ContextCompat.checkSelfPermission(activity, android.Manifest.permission.READ_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED) {
                    requestPermissions(arrayOf(android.Manifest.permission.READ_EXTERNAL_STORAGE), REQUEST_PERMISSIONS_FOR_FORCE_WALLPAPER)
                }
            }
            requireRestart()
        }
        if (binding.wallpaperScrollingForce.value)
            binding.wallpaperScrollingForce.visibility = View.VISIBLE
        binding.searchbarStyle.setOnClickListener {
            val intent = Intent(activity, SearchBarSettingsActivity::class.java)
            intent.putExtra("preferenceKey", "qsb_style")
            startActivity(intent)
        }
        binding.searchbarPlacement.onUserChanged = { oldStr, newStr ->
            val old = oldStr.parse(SearchBarPlacement.NONE)
            val new = newStr.parse(SearchBarPlacement.NONE)

            if (!new.isDock)
                Pref2.get().dock_qsb_appwidgetid.delete()
            if (new == SearchBarPlacement.WIDGET) {
                Pref3.searchbar_widget_ensure_onresume.value = 1
            } else if (old == SearchBarPlacement.WIDGET) {
                Pref3.searchbar_widget_ensure_onresume.value = -1
            }
            if (old != new && (old.requireRestart || new.requireRestart)) {
                requireRestart()
            }
        }

        val pm = activity.applicationContext.packageManager
        binding.autoAddApps.isChecked = Pref.Value.auto_add_shortcuts

        binding.autoAddApps.onUserChanged = { _, new ->
            if (Build.VERSION.SDK_INT < Build.VERSION_CODES.O) {
                pm.setComponentEnabledSetting(NewAppShortcutReceiver.COMPONENT_NAME, if (new) PackageManager.COMPONENT_ENABLED_STATE_ENABLED else PackageManager.COMPONENT_ENABLED_STATE_DISABLED, PackageManager.DONT_KILL_APP);
            }
            sharedPrefs.edit().putBoolean(Pref.Key.AUTO_ADD_SHORTCUTS, new).apply()
        }

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            binding.autoAddAppsPlayStore.visibility = View.GONE
        } else {
            try {
                pm.getPackageInfo(PackageNames.PLAY_STORE, 0)
                if (readPlayStoreAutoAddSetting()) {
                    binding.autoAddAppsPlayStore.setOnClickListener {
                        var intent = Intent(Intent.ACTION_MAIN)
                        intent.setClassName(PackageNames.PLAY_STORE, "com.google.android.finsky.activities.SettingsActivity")
                        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TASK)
                        try {
                            startActivity(intent)
                        } catch (ex: Exception) {
                            intent = Intent(Intent.ACTION_MAIN)
                            intent.addCategory(Intent.CATEGORY_LAUNCHER)
                            intent.setPackage(PackageNames.PLAY_STORE)
                            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TASK)
                            startActivity(intent)
                        }

                    }
                } else {
                    binding.autoAddAppsPlayStore.visibility = View.GONE
                }
            } catch (e: PackageManager.NameNotFoundException) {
                binding.autoAddAppsPlayStore.visibility = View.GONE
            }
        }

        binding.widgetOverlapWhenPlacing.setEnableCondition(Pref3.widget_overlap.asObservable)

        return binding.root
    }

    override fun onResume() {
        super.onResume()
        binding.iconLayout.summary = Pref3.desktop_cellspecs.value.toPreferenceSummaryString(activity)

        binding.grid.value = Pref.Value.desktop_grid_rows to Pref.Value.desktop_grid_cols
        binding.padding.value = Pref.Value.desktop_width_margin to Pref.Value.desktop_height_margin
        binding.autoAddApps.value = Pref.Value.auto_add_shortcuts
    }

    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<out String>, grantResults: IntArray) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        if (requestCode == REQUEST_PERMISSIONS_FOR_FORCE_WALLPAPER) {
            binding.wallpaperScrollingForce.isChecked = grantResults.isNotEmpty() && grantResults.all { it == PackageManager.PERMISSION_GRANTED }
            requireRestart()
        }
    }

    @SuppressLint("Recycle")
    private fun readPlayStoreAutoAddSetting(): Boolean {
        try {
            activity.contentResolver.query(
            Uri.parse("content://${PackageNames.PLAY_STORE}.addtohomescreen"), null, null, null, null)
            ?.use {
                    if (it.moveToNext())
                        return it.getInt(it.getColumnIndexOrThrow(Settings.NameValueTable.VALUE)) != 0
            }
        } catch (ex: Exception) {
            if (BuildConfig.DEBUG)
                ex.printStackTrace()
        }
        return true
    }

}
