package com.teslacoilsw.launcher.preferences.fragments

import android.app.FragmentTransaction
import android.content.Intent
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.core.view.forEach
import com.teslacoilsw.kotlin.bind
import com.teslacoilsw.launcher.NovaAction
import com.teslacoilsw.launcher.R
import com.teslacoilsw.launcher.databinding.SettingsDockBinding
import com.teslacoilsw.launcher.preferences.DockBackgroundSettingsActivity
import com.teslacoilsw.launcher.preferences.IconLayoutSettingsActivity
import com.teslacoilsw.launcher.preferences.Pref
import com.teslacoilsw.launcher.preferences.Pref3

class SettingsDock: NovaSettingsFragment() {
    private lateinit var binding : SettingsDockBinding

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup, savedInstanceState: Bundle?): View {
        binding = inflater.bind(R.layout.settings_dock, container, false)

        binding.enableDock.value = Pref.Value.dock_visible
        binding.enableDock.onUserChanged = { old, new ->
            binding.content.forEach {
                if (it != binding.enableDock)
                    it.isEnabled = new
            }
            binding.advanced.forEach {
                it.isEnabled = new
            }
            sharedPrefs.edit().putBoolean(Pref.Key.DOCK_ENABLED, new).apply()
            requireRestart()
        }

        binding.dockPages.onUserChanged = { _, new ->
            sharedPrefs.edit().putInt(Pref.Key.DOCK_PAGES, new).apply()
            requireRestart()
        }

        binding.iconLayout.setOnClickListener {
            val intent = Intent(activity, IconLayoutSettingsActivity::class.java)
            intent.putExtra("category", "DOCK")
            startActivity(intent)
        }

        if (sharedPrefs.getBoolean("big_grid_size", false))
            binding.dockIcons.maxCols = 16
        binding.dockIcons.onUserChanged = { _, new ->
            sharedPrefs.edit().putInt(Pref.Key.DOCK_GRID_COLS, new.second).apply()
            requireRestart()
        }

        binding.padding.onUserChanged = { _, it ->
            sharedPrefs.edit()
                    .putString(Pref.Key.DOCK_WIDTH_MARGIN, it.first.name)
                    .putString(Pref.Key.DOCK_HEIGHT_MARGIN, it.second.name)
                    .apply()
            requireRestart()
        }

        binding.dockAutoClose.setEnableCondition(Pref3.dock_overlay.asObservable)

        binding.dockBackground.setOnClickListener {
            val intent = Intent(activity, DockBackgroundSettingsActivity::class.java)
            startActivity(intent)
        }

        return binding.root
    }

    override fun onResume() {
        super.onResume()
        binding.enableDock.value = Pref.Value.dock_visible
        binding.enableDock.onUserChanged?.invoke(Pref.Value.dock_visible, Pref.Value.dock_visible)
        binding.dockPages.value = Pref.Value.dock_pages_count
        binding.dockIcons.value = 1 to Pref.Value.dock_grid_cols
        binding.iconLayout.summary = Pref3.dock_cellspecs.value.toPreferenceSummaryString(activity)
        binding.padding.value = Pref.Value.dock_width_margin to Pref.Value.dock_height_margin
        binding.dockPages.value = Pref.Value.dock_pages_count


        val gestureName = SettingsGestures.getPreferenceForAction(NovaAction.TOGGLE_DOCK)
        if (gestureName == null) {
            binding.hintGesture.visibility = View.GONE
        } else {
            binding.hintGesture.visibility = View.VISIBLE
            if (gestureName.iconId != 0)
                binding.hintGesture.setWidgetIconRes(gestureName.iconId)
            else
                binding.hintGesture.widgetIcon = null
            binding.hintGesture.title = getString(R.string.preference_gesture_configured_toggle, getString(gestureName.name))

            binding.hintGesture.setOnClickListener {
                val f = SettingsGestures()
                f.arguments = Bundle().apply {
                    putBundle("intentArgs", Bundle().apply {
                        putInt("targetPref", gestureName.prefId)
                    })
                }

                (activity).fragmentManager.beginTransaction()
                        .replace(R.id.fragment_container, f, "SETTINGS_FRAGMENT")
                        .setTransition(FragmentTransaction.TRANSIT_FRAGMENT_OPEN)
                        .addToBackStack(null)
                        .commit()
            }

        }

    }


}