package com.teslacoilsw.launcher.preferences.fragments

import android.content.Intent
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.core.graphics.alpha
import com.teslacoilsw.kotlin.bind
import com.teslacoilsw.launcher.R
import com.teslacoilsw.launcher.databinding.SettingsDrawerBinding
import com.teslacoilsw.launcher.drawer.drawergroups.DrawerGroupModel
import com.teslacoilsw.launcher.drawer.drawergroups.DrawerTabSetupActivity
import com.teslacoilsw.launcher.drawer.drawergroups.GroupAppListActivity
import com.teslacoilsw.launcher.preferences.IconLayoutSettingsActivity
import com.teslacoilsw.launcher.preferences.Pref
import com.teslacoilsw.launcher.preferences.Pref2
import com.teslacoilsw.launcher.preferences.Pref3
import com.teslacoilsw.launcher.preferences.SearchBarSettingsActivity
import com.teslacoilsw.launcher.preferences.widget.ScrollEffectPreference
import com.teslacoilsw.shared.colorpicker.ColorUtil

class SettingsDrawer: NovaSettingsFragment() {
    private lateinit var binding: SettingsDrawerBinding
    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup, savedInstanceState: Bundle?): View {
        binding = inflater.bind(R.layout.settings_drawer, container, false)

        binding.iconLayout.setOnClickListener {
            val intent = Intent(activity, IconLayoutSettingsActivity::class.java)
            intent.putExtra("category", "DRAWER")
            startActivity(intent)
        }
        binding.drawerStyle.onUserChanged = { old, new ->
            sharedPrefs.edit().putString(Pref.Key.DRAWER_STYLE, new).apply()
            updateHorizontalScrollOptions()
            requireRestart()
        }
        binding.backgroundColor.onUserChanged = { _, new ->
            saveBackgroundColor(255 - binding.backgroundTransparency.value, new)
        }
        binding.backgroundTransparency.onUserChanged = { _, new ->
            saveBackgroundColor(255 - new, binding.backgroundColor.value)
        }
        binding.cardBackground.onUserChanged = { old, new ->
            updateBackgroundColorPref(new)
            requireRestart()
        }

        binding.predictiveApps.onUserChanged = { old, new ->
            sharedPrefs.edit().putBoolean(Pref.Key.DRAWER_PREDICTIVE_APPS, new).apply()
            requireRestart()
        }

        binding.topBar.onUserChanged = { old, new ->
            when (new) {
                "SEARCH" -> {
                    binding.tabBarOptions.visibility = View.GONE
                    binding.searchBarOptions.visibility = View.VISIBLE
                    Pref3.drawer_enable_tabs.value = false
                    Pref3.drawer_searchbar.value = true
                }
                "TABS" -> {
                    binding.tabBarOptions.visibility = View.VISIBLE
                    binding.searchBarOptions.visibility = View.GONE
                    Pref3.drawer_enable_tabs.value = true
                    Pref3.drawer_searchbar.value = false
                }
                else -> {
                    binding.tabBarOptions.visibility = View.GONE
                    binding.searchBarOptions.visibility = View.GONE
                    Pref3.drawer_enable_tabs.value = false
                    Pref3.drawer_searchbar.value = false
                }
            }
            requireRestart()
        }
        binding.topBar.value = when {
            Pref3.drawer_searchbar.value -> "SEARCH"
            Pref3.drawer_enable_tabs.value -> "TABS"
            else -> "NONE"
        }
        binding.topBar.onUserChanged?.invoke("", binding.topBar.value)
        binding.drawerTabStyle.onUserChanged = { old, new ->
            sharedPrefs.edit().putString(Pref.Key.DRAWER_TAB_STYLE, new).apply()
            requireRestart()
        }

        if (sharedPrefs.getBoolean("big_grid_size", false)) {
            binding.grid.maxCols = 16
            binding.grid.maxRows = 16
        }
        binding.grid.onUserChanged = { _, _ ->
            sharedPrefs.edit().putInt(Pref.Key.DRAWER_APP_GRID_ROWS, binding.grid.value.first)
                    .putInt(Pref.Key.DRAWER_APP_GRID_COLS, binding.grid.value.second)
                    .putInt(Pref.Key.DRAWER_APP_GRID_LAND_ROWS, binding.grid.landValue.first)
                    .putInt(Pref.Key.DRAWER_APP_GRID_LAND_COLS, binding.grid.landValue.second)
                    .apply()
            requireRestart()
        }

        binding.searchbarStyle.setOnClickListener {
            val intent = Intent(activity, SearchBarSettingsActivity::class.java)
            intent.putExtra("preferenceKey", Pref3.drawer_search_style.key)
            startActivity(intent)
        }

        binding.swipeToOpen.onUserChanged = { _, new ->
            Pref2.get().drawer.drawer_swipe_from_dock.set(new)
            requireRestart()
        }
        binding.swipeToOpenIndicator.setEnableCondition(Pref2.get().drawer.drawer_swipe_from_dock.asObservable())

        binding.drawerAnimation.setEnableCondition(Pref3.drawer_swipe_to_open.asObservable.map { !it })
        binding.drawerAnimation.onUserChanged = { _, new ->
            if (Pref.Value.animation_speed == Pref.Key.AnimationSpeed.FTL) {
                sharedPrefs.edit().putString(Pref.Key.ANIMATION_SPEED, Pref.Key.AnimationSpeed.NOVA.name).apply()
            }

            if (Pref.Key.DrawerAnimation.CIRCLE_CARD.name == new && !binding.cardBackground.isChecked) {
                binding.cardBackground.isChecked = true
            }

            sharedPrefs.edit().putString(Pref.Key.DRAWER_ANIMATION, new).apply()
        }

        updateHorizontalScrollOptions()

        if (!Pref.Value.nova_prime_enabled)
            binding.scrollEffect.entries = binding.scrollEffect.entries.subList(0, 3)
        fun scrollEffectAnim(workspace: ScrollEffectPreference.WorkspaceView) {
            var newPage = workspace.nextPage + 1
            if (newPage >= 2)
                newPage = 0
            workspace.snapToPage(newPage, 1500)
        }
        binding.scrollEffect.onBindDialog = { dialogContent ->
            dialogContent.findViewById<ScrollEffectPreference.WorkspaceView>(R.id.workspace).setOnClickListener {
                scrollEffectAnim(it as ScrollEffectPreference.WorkspaceView)
            }
        }
        binding.scrollEffect.onSelectedInDialog = { dialogContent, entry ->
            val workspace = dialogContent.findViewById<ScrollEffectPreference.WorkspaceView>(R.id.workspace)
            workspace.setScrollEffect(Pref.Key.ScrollEffectEnum.valueOf(entry.value))
            scrollEffectAnim(workspace)
        }

        binding.drawerFastScrollColor.onUserChanged = { _, new ->
            Pref3.drawer_fastscroll_color.value = new
            requireRestart()
        }

        binding.drawerMenuActions.onUserChanged = { _, new ->
            sharedPrefs.edit().putStringSet(Pref.Key.DRAWER_MENU_ACTION_SET, new.toSet()).apply()
            requireRestart()
        }
        binding.drawerMenuActions.value = Pref.Value.drawer_menu_action.map { it.name }.toSet()

        binding.hideApps.setOnClickListener {
            val intent = Intent(activity, GroupAppListActivity::class.java)
            startActivity(intent)
        }

        binding.drawerGroups.setOnClickListener {
            val intent = Intent(activity, DrawerTabSetupActivity::class.java)
            startActivity(intent)
        }

        refreshState()

        return binding.root
    }

    fun refreshState() {

        binding.iconLayout.summary = Pref3.drawer_cellspecs.value.toPreferenceSummaryString(activity)

        binding.drawerStyle.value = Pref.Value.drawer_style.name
        binding.topBar.value = when {
            Pref3.drawer_searchbar.value -> "SEARCH"
            Pref3.drawer_enable_tabs.value -> "TABS"
            else -> "NONE"
        }
        binding.drawerTabStyle.value = Pref.Value.drawer_tab_style.name

        binding.grid.value = Pref.Value.drawer_app_grid_rows to Pref.Value.drawer_app_grid_cols
        binding.grid.landValue = Pref.Value.drawer_app_grid_land_rows to Pref.Value.drawer_app_grid_land_cols
        binding.swipeToOpen.value = Pref2.get().drawer.drawer_swipe_from_dock.get()!!
        updateBackgroundColorPref(Pref3.drawer_cards.value)

        if (Pref3.night_mode_auto_drawer.value) {
            binding.nightModeHint.setWidgetIconRes(if (Pref3.night_mode.value.isAuto) R.drawable.ic_night_auto else R.drawable.ic_night_on)
            binding.nightModeHint.visibility = View.VISIBLE

            binding.nightModeHint.setOnClickListener {
//                val f = SettingsNightMode()
//                f.arguments = Bundle().apply {
//                    putBundle("intentArgs", Bundle().apply {
//                        putInt("targetPref", R.id.nightModeDrawer)
//                    })
//                }

//                (activity).fragmentManager.beginTransaction()
//                        .replace(R.id.fragment_container, f, "SETTINGS_FRAGMENT")
//                        .setTransition(FragmentTransaction.TRANSIT_FRAGMENT_OPEN)
//                        .addToBackStack(null)
//                        .commit()
            }
        }

        binding.drawerFastScrollColor.value = Pref3.drawer_fastscroll_color.value
        binding.drawerAnimation.value = Pref.Value.drawerAnimation.name
        binding.predictiveApps.value = Pref.Value.drawer_predictive_apps

        updateHorizontalScrollOptions()
    }

    override fun onResume() {
        super.onResume()
        refreshState()
    }

    private fun updateHorizontalScrollOptions() {
        DrawerGroupModel.get().load(activity.contentResolver)
        val hasTabs = DrawerGroupModel.get().tabContent.size > 1

        if (Pref.Value.drawer_style == Pref.Key.DrawerStyle.HORIZONTAL_PAGINATED || hasTabs) {
            binding.horizontalScrollOptions.visibility = View.VISIBLE
        } else {
            binding.horizontalScrollOptions.visibility = View.GONE
        }
    }

    private fun updateBackgroundColorPref(cardBackground: Boolean) {
        if (cardBackground) {
            binding.backgroundColor.value = Pref3.drawer_card_color.value
            binding.backgroundTransparency.value = 255 - Pref3.drawer_card_color.value.alpha
        } else {
            binding.backgroundColor.value = Pref3.drawer_background_color.value
            binding.backgroundTransparency.value = 255 - Pref3.drawer_background_color.value.alpha
        }
    }

    private fun saveBackgroundColor(alpha: Int, color: Int) {
        val newColor = ColorUtil.argb(alpha, color)
        if (binding.cardBackground.isChecked) {
            Pref3.drawer_card_color.value = newColor
        } else {
            Pref3.drawer_background_color.value = newColor
        }
    }
}