package com.teslacoilsw.launcher.preferences.fragments

import android.content.Intent
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.drawable.Drawable
import android.os.Bundle
import android.text.Spannable
import android.text.SpannableString
import android.text.SpannableStringBuilder
import android.text.style.ImageSpan
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import aosp.android.graphics.drawable.InsetDrawableProportional
import com.android.launcher3.BubbleTextView
import com.android.launcher3.FastBitmapDrawable
import com.android.launcher3.FolderIcon
import com.teslacoilsw.colornames.ColorNames
import com.teslacoilsw.kotlin.bind
import com.teslacoilsw.kotlin.lazy1
import com.teslacoilsw.kotlin.ucfirst
import com.teslacoilsw.launcher.FolderIconHelper
import com.teslacoilsw.launcher.R
import com.teslacoilsw.launcher.databinding.SettingsFolderBinding
import com.teslacoilsw.launcher.preferences.FolderIconSettingsActivity
import com.teslacoilsw.launcher.preferences.FolderWindowSettingsActivity
import com.teslacoilsw.launcher.preferences.IconLayoutSettingsActivity
import com.teslacoilsw.launcher.preferences.Pref3
import com.teslacoilsw.launcher.preferences.fancyprefs.color.ColorStateDrawable
import com.teslacoilsw.launcher.util.DpUtil
import com.teslacoilsw.matcompat.TintCompat
import com.teslacoilsw.shared.colorpicker.ColorUtil
import kotlin.math.roundToInt

class SettingsFolder: NovaSettingsFragment() {
    private lateinit var binding: SettingsFolderBinding

    private val folderIconHelper by lazy1 { FolderIconHelper(activity) }

    private val folderIconSize by lazy1 { DpUtil.getPixelSize(48) }
    private val appIconSize by lazy1 { (folderIconSize/ FolderIcon.FOLDER_ICON_SCALE).toInt() }

    private val previewBitmap by lazy1 { Bitmap.createBitmap(folderIconSize, folderIconSize, Bitmap.Config.ARGB_8888) }
    private val previewCanvas = Canvas()
    private val previewItems by lazy1 { listOf(
            BubbleTextView(activity).apply { applyCompoundDrawables(FastBitmapDrawable(BitmapFactory.decodeResource(resources, R.drawable.ic_lptheme_settings)).apply { setBounds(0, 0, appIconSize, appIconSize) }) },
            BubbleTextView(activity).apply { applyCompoundDrawables(FastBitmapDrawable(BitmapFactory.decodeResource(resources, R.drawable.ic_lptheme_email)).apply { setBounds(0, 0, appIconSize, appIconSize) }) },
            BubbleTextView(activity).apply { applyCompoundDrawables(FastBitmapDrawable(BitmapFactory.decodeResource(resources, R.drawable.ic_lptheme_clock)).apply { setBounds(0, 0, appIconSize, appIconSize) }) },
            BubbleTextView(activity).apply { applyCompoundDrawables(FastBitmapDrawable(BitmapFactory.decodeResource(resources, R.drawable.ic_lptheme_sms)).apply { setBounds(0, 0, appIconSize, appIconSize) }) }
    ) }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup, savedInstanceState: Bundle?): View {
        binding = inflater.bind(R.layout.settings_folder, container, false)

        binding.iconLayout.setOnClickListener {
            val intent = Intent(activity, IconLayoutSettingsActivity::class.java)
            intent.putExtra("category", "FOLDER")
            startActivity(intent)
        }

        binding.folderIconAppearance.setOnClickListener {
            val i = Intent()
            i.setClass(activity, FolderIconSettingsActivity::class.java)
            startActivity(i)
        }

        binding.folderWindowStyle.setOnClickListener {
            val i = Intent()
            i.setClass(activity, FolderWindowSettingsActivity::class.java)
            startActivity(i)
        }

        return binding.root
    }

    override fun onResume() {
        super.onResume()
        binding.folderIconAppearance.widgetIcon = generateFolderIconPreview()

        val inset = 0.15f
        val windowConfig = Pref3.folder_window_config.value
        val size = (resources.displayMetrics.scaledDensity * (1f+inset)*14f).roundToInt()
        val colorName = if (windowConfig.backgroundAlpha > 100) ColorNames.getBestName(windowConfig.backgroundColor).name else "Transparent"

        binding.folderWindowStyle.summary = SpannableStringBuilder().append(windowConfig.style.name.ucfirst())
                .append(".  ")
                .append(windowConfig.scrollDirection.name.ucfirst())
                .append(".  ")
                .append(SpannableString("# " + colorName).apply {
                    setSpan(ImageSpan(InsetDrawableProportional(ColorStateDrawable(resources, ColorUtil.argb(windowConfig.backgroundAlpha, windowConfig.backgroundColor), .5f), inset, 0f, 0f, inset).apply { setBounds(0, 0, size, size) }, ImageSpan.ALIGN_BOTTOM), 0, 1, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE)
                })
        binding.iconLayout.summary = Pref3.folder_cellspecs.value.toPreferenceSummaryString(activity)
    }

    private fun generateFolderIconPreview(): Drawable {
        folderIconHelper.setConfigFromPreferences(activity)
        previewBitmap.eraseColor(Color.TRANSPARENT)
        previewCanvas.setBitmap(previewBitmap)

        folderIconHelper.computePreviewDrawingParams(folderIconSize, appIconSize, folderIconSize)
        val windowColor = TintCompat.getThemeAttrColor(activity, android.R.attr.windowBackground)
        folderIconHelper.drawFolderIcon(previewCanvas, previewItems, null, true, false, null, windowColor, false)
        previewCanvas.setBitmap(null)
        return FastBitmapDrawable(previewBitmap).apply { setBounds(0, 0, folderIconSize, folderIconSize) }
    }


}