package com.teslacoilsw.launcher.preferences.fragments

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import android.provider.Settings
import android.view.LayoutInflater
import android.view.View
import android.view.ViewConfiguration
import android.view.ViewGroup
import com.teslacoilsw.kotlin.bind
import com.teslacoilsw.kotlin.forEach
import com.teslacoilsw.launcher.ChooseActionIntentActivity
import com.teslacoilsw.launcher.NovaAction
import com.teslacoilsw.launcher.R
import com.teslacoilsw.launcher.databinding.SettingsGesturesBinding
import com.teslacoilsw.launcher.preferences.Pref
import com.teslacoilsw.launcher.preferences.Pref3
import com.teslacoilsw.launcher.preferences.fancyprefs.FancyPrefIntentView
import com.teslacoilsw.launcher.util.NamedIntent
import com.teslacoilsw.shared.util.ReflectionHelper

private val HAS_CM_LONG_MENU_ACTION = ReflectionHelper.getStaticString(Settings.System::class.java, "KEY_MENU_LONG_PRESS_ACTION", null) != null
private const val REQUEST_CREATE_SHORTCUT = 1

class SettingsGestures : NovaSettingsFragment() {

    private lateinit var binding: SettingsGesturesBinding

    private var pendingPreferenceViewId: Int = 0

    private val intentPickerClickListener = View.OnClickListener {
        val pref = it as FancyPrefIntentView
        pendingPreferenceViewId = it.id
        val intent = Intent(activity, ChooseActionIntentActivity::class.java)
        intent.putExtra(ChooseActionIntentActivity.EXTRA_TITLE, pref.title)
        startActivityForResult(intent, REQUEST_CREATE_SHORTCUT)
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup, savedInstanceState: Bundle?): View {
        binding = inflater.bind(R.layout.settings_gestures, container, false)

        var showMenuKey = ViewConfiguration.get(activity).hasPermanentMenuKey()

        /* Don't show if we're on CM and not using a navbar, as CM handles long-press itself */
        if (showMenuKey && HAS_CM_LONG_MENU_ACTION != null) {
            showMenuKey = false
        } else if (usedMenuKey) {
            showMenuKey = true
        }

        if (!showMenuKey)
            binding.longPressMenu.visibility = View.GONE

        (binding.root as ViewGroup).forEachFancyPrefIntentRecursive {
            setOnClickListener(intentPickerClickListener)
        }
        return binding.root
    }

    private fun ViewGroup.forEachFancyPrefIntentRecursive(block: FancyPrefIntentView.() -> Unit) {
        forEach {
            if (it is FancyPrefIntentView) {
                it.block()
            } else if (it is ViewGroup) {
                it.forEachFancyPrefIntentRecursive(block)
            }
        }
    }

    override fun onSaveInstanceState(outState: Bundle) {
        super.onSaveInstanceState(outState)
        if (pendingPreferenceViewId != 0)
            outState.putInt("prefId", pendingPreferenceViewId)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == REQUEST_CREATE_SHORTCUT && resultCode == Activity.RESULT_OK) {
            val ni = NamedIntent.fromActivityResult(data) ?: NamedIntent.NONE
            if (pendingPreferenceViewId != 0) {
                val pref = view.findViewById<FancyPrefIntentView>(pendingPreferenceViewId)
                pref.value = ni
                pendingPreferenceViewId = 0
            }
        }
    }

    companion object {
        @JvmStatic
        var usedMenuKey = false

        private fun nameForGesture(prefKey: String): GesturePrefInfo {
            return when (prefKey) {
                Pref.Key.GESTURE_SWIPE_DOWN -> GesturePrefInfo(R.string.gesture_swipe_down, R.drawable.ic_gesture_down, R.id.swipeDown)
                Pref.Key.GESTURE_SWIPE_UP -> GesturePrefInfo(R.string.gesture_swipe_up, R.drawable.ic_gesture_up, R.id.swipeUp)
                Pref.Key.GESTURE_TWO_FINGER_SCROLL_DOWN -> GesturePrefInfo(R.string.gesture_swipe_down_two, R.drawable.ic_gesture_twodown, R.id.twoDown)
                Pref.Key.GESTURE_TWO_FINGER_SCROLL_UP -> GesturePrefInfo(R.string.gesture_swipe_up_two, R.drawable.ic_gesture_twoup, R.id.twoUp)
                Pref.Key.GESTURE_ROTATE_CW -> GesturePrefInfo(R.string.gesture_rotate_cw, R.drawable.ic_gesture_clockwise, R.id.rotateCW)
                Pref.Key.GESTURE_ROTATE_CCW -> GesturePrefInfo(R.string.gesture_rotate_ccw, R.drawable.ic_gesture_counterclockwise, R.id.rotateCCW)
                Pref.Key.GESTURE_PINCH -> GesturePrefInfo(R.string.gesture_pinch_in, R.drawable.ic_gesture_pinchin, R.id.pinchIn)
                Pref.Key.GESTURE_PINCH_OUT -> GesturePrefInfo(R.string.gesture_pinch_out, R.drawable.ic_gesture_pinchout, R.id.pinchOut)
                Pref.Key.GESTURE_DOUBLE_TAP -> GesturePrefInfo(R.string.gesture_double_tap, R.drawable.ic_gesture_doubletap, R.id.doubleTap)
                Pref.Key.HOME_BUTTON_INTENT -> GesturePrefInfo(R.string.preference_home_button, R.drawable.ic_gesture_home, R.id.homeButton)
                Pref3.gesture_long_menu.key -> GesturePrefInfo(R.string.preference_longpress_menu, R.drawable.ic_gesture_menu, R.id.longPressMenu)
                Pref.Key.GESTURE_DOUBLE_TAP_SWIPE_DOWN -> GesturePrefInfo(R.string.gesture_double_tap_swipe_down, R.drawable.ic_gesture_doubletap_swipe_down, R.id.doubleTapDown)
                Pref.Key.GESTURE_DOUBLE_TAP_SWIPE_UP -> GesturePrefInfo(R.string.gesture_double_tap_swipe_up, R.drawable.ic_gesture_doubletap_swipe_up, R.id.doubleTapUp)
                else -> throw RuntimeException("$prefKey is not a valid gesture key")
            }
        }

        val prefs = arrayOf(
                Pref3.gesture_home_button,
                Pref3.gesture_long_menu,
                Pref3.gesture_swipe_up,
                Pref3.gesture_swipe_down,
                Pref3.gesture_pinch_in,
                Pref3.gesture_pinch_out,
                Pref3.gesture_swipe_two_up,
                Pref3.gesture_swipe_two_down,
                Pref3.gesture_double_tap,
                Pref3.gesture_rotate_ccw,
                Pref3.gesture_rotate_cw,
                Pref3.gesture_double_tap_swipe_up,
                Pref3.gesture_double_tap_swipe_down
        )
        
        fun getPreferenceForAction(action: NovaAction): GesturePrefInfo? {
            for (p in prefs) {
                if (NovaAction.valueOf(p.value.intent) == action)
                    return nameForGesture(p.key)
            }
            return null
        }

    }
}

data class GesturePrefInfo constructor(val name: Int, val iconId: Int, val prefId: Int)
