package com.teslacoilsw.launcher.preferences.fragments

import android.Manifest
import android.annotation.SuppressLint
import android.app.ProgressDialog
import android.content.Intent
import android.content.pm.PackageManager
import android.content.res.Resources
import android.net.Uri
import android.os.AsyncTask
import android.os.Build
import android.os.Bundle
import android.os.Environment
import android.provider.Settings
import android.util.DisplayMetrics
import android.util.TypedValue
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import androidx.core.view.isVisible
import com.android.launcher3.LauncherAppState
import com.android.launcher3.LauncherModel
import com.teslacoilsw.kotlin.bind
import com.teslacoilsw.kotlin.hasPermission
import com.teslacoilsw.launcher.R
import com.teslacoilsw.launcher.bitmaputils.BitmapUtils
import com.teslacoilsw.launcher.databinding.SettingsLabsBinding
import com.teslacoilsw.launcher.drawer.WidgetPageFactory
import com.teslacoilsw.launcher.preferences.Pref2
import com.teslacoilsw.launcher.preferences.Pref3
import com.teslacoilsw.launcher.preferences.fancyprefs.FancyPrefCheckableView
import com.teslacoilsw.launcher.quicksearchbar.SearchBarStyle
import com.teslacoilsw.shared.util.FileSlurp
import java.io.FileOutputStream
import java.io.IOException
import java.util.zip.ZipEntry
import java.util.zip.ZipOutputStream

class SettingsLabs: NovaSettingsFragment() {
    private val permissionPrefs = mutableMapOf<String, FancyPrefCheckableView>()
    private lateinit var binding: SettingsLabsBinding

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup, savedInstanceState: Bundle?): View {
        binding = inflater.bind(R.layout.settings_labs, container, false)

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            binding.weatherInSearch.isChecked = activity.hasPermission(Manifest.permission.ACCESS_FINE_LOCATION) && Pref3.qsb_style.value.hasWeather()
            binding.weatherInSearch.onUserChanged = { _, new ->
                if (new) {
                    if (!activity.hasPermission(Manifest.permission.ACCESS_FINE_LOCATION)) {
                        requestPermissions(arrayOf(Manifest.permission.ACCESS_FINE_LOCATION), 1)
                        binding.weatherInSearch.isChecked = false
                    } else {
                        updateWeatherInSearchbar(true)
                    }
                } else {
                    updateWeatherInSearchbar(false)
                }
            }
        } else {
            binding.weatherInSearch.isVisible = false
        }

        binding.debugDialog.setOnClickListener {
            DebugInfoDialog.create(activity).show()
        }

        binding.bypassSystemTheme.value = Pref2.get().bypass_system_theme.get()!!
        binding.bypassSystemTheme.onUserChanged = { _, new ->
            clearIconCache()
            Pref2.get().bypass_system_theme.set(new)
        }
        binding.bigGridSize.value = sharedPrefs.getBoolean("big_grid_size", false)
        binding.bigGridSize.onUserChanged = { _, new ->
            sharedPrefs.edit().putBoolean("big_grid_size", new).apply()
        }

        binding.clearIconCache.setOnClickListener {
            clearIconCache()
            Toast.makeText(activity, "Cleared icon cache", Toast.LENGTH_SHORT).show()
        }

        binding.dumpIcons.setOnClickListener {
            val d = ProgressDialog(activity)
            d.setProgressStyle(ProgressDialog.STYLE_SPINNER)
            d.setMessage(resources.getString(R.string.preparing_theme))
            d.setCancelable(false)
            d.show()
            object : AsyncTask<Void, Void, Void>() {

                @SuppressLint("InlinedApi")
                override fun doInBackground(vararg params: Void): Void? {
                    try {
                        val buffer = ByteArray(4096)
                        val typedValue = TypedValue()
                        val zp = ZipOutputStream(FileOutputStream(Environment.getExternalStorageDirectory().path + "/novaIconExport.zip"))

                        var ze = ZipEntry("README.txt")
                        zp.putNextEntry(ze)
                        zp.write(("This is a dump of the icons from your system. You can use it as a base for creating a new theme.\n" + "It includes all the appropriate component names in res/xml/drawables.xml and their respective icons at the highest density provided by the APK").toByteArray())
                        zp.closeEntry()

                        ze = ZipEntry("res/xml/appfilter.xml")
                        zp.putNextEntry(ze)
                        val sb = StringBuilder("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n")
                        sb.append("<resources>\n")
                        zp.write(sb.toString().toByteArray())
                        sb.delete(0, sb.length)

                        val pm = LauncherAppState.getInstance().context.packageManager
                        val list = pm.queryIntentActivities(Intent(Intent.ACTION_MAIN).addCategory(Intent.CATEGORY_LAUNCHER), 0)
                        for (ri in list) {
                            sb.append("    ").append("<item component=\"ComponentInfo{")
                                    .append(ri.activityInfo.packageName).append("/")
                                    .append(ri.activityInfo.name).append("}\"")
                                    .append(" drawable=\"ic_").append(ri.activityInfo.packageName.replace(':', '_'))
                                    .append("___").append(ri.activityInfo.name.replace(':', '_'))
                                    .append("\" />\n")
                            zp.write(sb.toString().toByteArray())
                            sb.delete(0, sb.length)
                        }
                        zp.write("</resources>\n".toByteArray())
                        zp.closeEntry()

                        for (ri in list) {
                            try {
                                val `is` = BitmapUtils.getBitmapInputStream(pm.getResourcesForApplication(ri.activityInfo.packageName), ri.iconResource, DisplayMetrics.DENSITY_XXXHIGH, typedValue)
                                        ?: continue

                                sb.delete(0, sb.length)
                                sb.append("res/")
                                if (typedValue.density <= DisplayMetrics.DENSITY_LOW) {
                                    sb.append("drawable-ldpi/")
                                } else if (typedValue.density <= DisplayMetrics.DENSITY_MEDIUM) {
                                    sb.append("drawable-mdpi/")
                                } else if (typedValue.density <= DisplayMetrics.DENSITY_HIGH) {
                                    sb.append("drawable-hdpi/")
                                } else if (typedValue.density <= DisplayMetrics.DENSITY_XHIGH) {
                                    sb.append("drawable-xhdpi/")
                                } else if (typedValue.density <= DisplayMetrics.DENSITY_XXHIGH) {
                                    sb.append("drawable-xxhdpi/")
                                } else {
                                    sb.append("drawable-xxxhdpi/")
                                }
                                sb.append("ic_").append(ri.activityInfo.packageName.replace(':', '_')).append("___")
                                        .append(ri.activityInfo.name.replace(':', '_')).append(".png")

                                ze = ZipEntry(sb.toString())
                                zp.putNextEntry(ze)

                                var count = `is`.read(buffer)
                                while (count != -1) {
                                    zp.write(buffer, 0, count)
                                    count = `is`.read(buffer)
                                }
                                zp.closeEntry()
                            } catch (ex: Resources.NotFoundException) {
                                ex.printStackTrace()
                            } catch (ex: PackageManager.NameNotFoundException) {
                                ex.printStackTrace()
                            }

                        }

                        zp.close()

                    } catch (ex: IOException) {
                        ex.printStackTrace()
                    }

                    return null
                }

                public override fun onPostExecute(result: Void) {
                    d.dismiss()
                    Toast.makeText(activity, "Exported icons and activity names to /sdcard/novaIconExport.zip", Toast.LENGTH_LONG).show()
                }
            }.execute()
        }

        try {
            val leakActivity = Class.forName("com.squareup.leakcanary.internal.DisplayLeakActivity")
            binding.leakCanary.isVisible = true
            binding.leakCanary.setOnClickListener {
                startActivity(Intent(activity, leakActivity))
            }
        } catch (e: ClassNotFoundException) {
        }

        permissionPrefs.clear()
        binding.permLocation.bindPermission(android.Manifest.permission.ACCESS_FINE_LOCATION)
        binding.permPhone.bindPermission(android.Manifest.permission.CALL_PHONE)
        binding.permStorage.bindPermission(android.Manifest.permission.READ_EXTERNAL_STORAGE)

        return binding.root
    }

    private fun FancyPrefCheckableView.bindPermission(permission: String) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            permissionPrefs[permission] = this
            isChecked = activity.hasPermission(permission)
            setOnClickListener {
                if (!isChecked) {
                    requestPermissions(arrayOf(permission), 2)
                } else {
                    val intent = Intent(Settings.ACTION_APPLICATION_DETAILS_SETTINGS)
                    val uri = Uri.fromParts("package", activity.packageName, null)
                    intent.data = uri
                    startActivity(intent)
                }
            }
        } else {
            binding.permissionGroup.isVisible = false
            isVisible = false
        }
    }

    override fun onResume() {
        super.onResume()
        for (p in permissionPrefs) {
            p.value.isChecked = activity.hasPermission(p.key)
        }
    }

    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<out String>?, grantResults: IntArray?) {
        if (requestCode == 1) {
            if (permissions?.size == 1 && permissions[0] == Manifest.permission.ACCESS_FINE_LOCATION && grantResults!![0] == PackageManager.PERMISSION_GRANTED) {
                updateWeatherInSearchbar(true)
                binding.weatherInSearch.isChecked = true
            }
        } else if (requestCode == 2) {

        }
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
    }

    private fun updateWeatherInSearchbar(weather: Boolean) {
        val s = Pref3.qsb_style.value
        val flags = if (weather) s.contentFlags or SearchBarStyle.CONTENT_WEATHER else s.contentFlags and SearchBarStyle.CONTENT_WEATHER.inv()
        Pref3.qsb_style.value = SearchBarStyle(s.logoStyle, s.bgStyle, s.bgColor, flags)
    }

    private fun clearIconCache() {
        LauncherModel.deleteCachedIcons(activity)
        FileSlurp.deleteDirectory(WidgetPageFactory.getWidgetPreviewCacheDir(activity))
        LauncherAppState.forceModelReload()
    }
}