package com.teslacoilsw.launcher.preferences.fragments

import android.app.AlertDialog
import android.app.FragmentTransaction
import android.content.Intent
import android.content.pm.PackageManager
import android.os.Build
import android.os.Bundle
import android.provider.Settings
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.CheckBox
import androidx.core.view.isVisible
import com.android.launcher3.LauncherAppState
import com.android.launcher3.LauncherModel
import com.teslacoilsw.kotlin.bind
import com.teslacoilsw.launcher.NovaAction
import com.teslacoilsw.launcher.R
import com.teslacoilsw.launcher.databinding.SettingsLookfeelBinding
import com.teslacoilsw.launcher.icon.NovaAdaptiveIconDrawable
import com.teslacoilsw.launcher.preferences.AdaptiveIconSettingsActivity
import com.teslacoilsw.launcher.preferences.PopupItemVisibility
import com.teslacoilsw.launcher.preferences.PopupItems
import com.teslacoilsw.launcher.preferences.Pref
import com.teslacoilsw.launcher.preferences.Pref2
import com.teslacoilsw.launcher.preferences.Pref3
import com.teslacoilsw.launcher.util.OEMUtils
import com.teslacoilsw.launcher.util.StatusBar.HideClock
import com.teslacoilsw.shared.util.rewthelper.RewtHelper
import rx.Observable
import rx.android.schedulers.AndroidSchedulers
import rx.schedulers.Schedulers
import rx.subscriptions.CompositeSubscription

class SettingsLookFeel: NovaSettingsFragment() {
    private var subscription: CompositeSubscription? = null
    private lateinit var binding: SettingsLookfeelBinding

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup, savedInstanceState: Bundle?): View {
        val hasRoot = RewtHelper.hasSuperuserBin()

        binding = inflater.bind(R.layout.settings_lookfeel, container, false)
        binding.iconStyle.setOnClickListener {
            val intent = Intent(activity, AdaptiveIconSettingsActivity::class.java)
            startActivity(intent)
        }
        binding.iconStyle.setOnLongClickListener {
            LauncherAppState.getInstance().iconCache.flushCustomIcons()
            LauncherAppState.getInstance().iconCache.setIconThemeFromConfig()
            LauncherModel.deleteCachedIcons(activity)
            LauncherAppState.forceModelReload()
            true
        }
        binding.animationSpeed.onUserChanged = { _, newValue ->
            val cr = activity.contentResolver
            if (Pref.Key.AnimationSpeed.valueOf(newValue) != Pref.Key.AnimationSpeed.FTL) {
                for (key in arrayOf(Settings.Global.WINDOW_ANIMATION_SCALE, Settings.Global.ANIMATOR_DURATION_SCALE, Settings.Global.TRANSITION_ANIMATION_SCALE)) {
                    if (Settings.Global.getFloat(cr, key, 1f) <= 0) {
                        showDevAnimationSpeedDialog()
                        break
                    }
                }
            }
        }
        binding.samsungHideNavbar.isVisible = OEMUtils.hasSamsungS8NavBar()
        binding.hideClock.isVisible = HideClock.hasSystemPriv() || HideClock.hasAdbPriv() || (Build.VERSION.SDK_INT < Build.VERSION_CODES.O && hasRoot)
        binding.hideClock.onUserChanged = { _, newValue ->
            if (newValue != HideClock.DISABLED.name) {
                if (!HideClock.hasSystemPriv() && !HideClock.hasAdbPriv())
                    RewtHelper.checkSu()
            }
        }

        binding.popupMenuStyle.onBindDialog = {
            val p = Pref3.popup_menu_item_visibility.value
            val uninstallDesktop = it.findViewById<CheckBox>(R.id.uninstall_desktop)
            val uninstallDrawer = it.findViewById<CheckBox>(R.id.uninstall_drawer)

            it.findViewById<View>(R.id.uninstall).setOnClickListener {
                if (uninstallDesktop.isChecked && uninstallDrawer.isChecked) {
                    uninstallDesktop.isChecked = false
                    uninstallDrawer.isChecked = false
                } else {
                    uninstallDesktop.isChecked = true
                    uninstallDrawer.isChecked = true
                }
            }
            uninstallDesktop.isChecked = p.uninstall.desktop
            uninstallDrawer.isChecked = p.uninstall.drawer


            val playDesktop = it.findViewById<CheckBox>(R.id.play_desktop)
            val playDrawer = it.findViewById<CheckBox>(R.id.play_drawer)

            it.findViewById<View>(R.id.play).setOnClickListener {
                if (playDesktop.isChecked && playDrawer.isChecked) {
                    playDesktop.isChecked = false
                    playDrawer.isChecked = false
                } else {
                    playDesktop.isChecked = true
                    playDrawer.isChecked = true
                }
            }
            playDesktop.isChecked = p.playStore.desktop
            playDrawer.isChecked = p.playStore.drawer


            val relaunchDesktop = it.findViewById<CheckBox>(R.id.relaunch_desktop)
            val relaunchDrawer = it.findViewById<CheckBox>(R.id.relaunch_drawer)

            it.findViewById<View>(R.id.relaunch).setOnClickListener {
                if (relaunchDesktop.isChecked && relaunchDrawer.isChecked) {
                    relaunchDesktop.isChecked = false
                    relaunchDrawer.isChecked = false
                } else {
                    relaunchDesktop.isChecked = true
                    relaunchDrawer.isChecked = true
                }
            }
            relaunchDesktop.isChecked = p.relaunch.desktop
            relaunchDrawer.isChecked = p.relaunch.drawer
        }

        binding.popupMenuStyle.onPositive = {
            val uninstall = PopupItemVisibility(it.findViewById<CheckBox>(R.id.uninstall_desktop).isChecked,
                        it.findViewById<CheckBox>(R.id.uninstall_drawer).isChecked)
            val play = PopupItemVisibility(it.findViewById<CheckBox>(R.id.play_desktop).isChecked,
                    it.findViewById<CheckBox>(R.id.play_drawer).isChecked)
            val relaunch = PopupItemVisibility(it.findViewById<CheckBox>(R.id.relaunch_desktop).isChecked,
                    it.findViewById<CheckBox>(R.id.relaunch_drawer).isChecked)

            Pref3.popup_menu_item_visibility.value = PopupItems(uninstall, play, relaunch)
            updatePopupMenuSummary()
        }
        updatePopupMenuSummary()

        return binding.root
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        subscription = CompositeSubscription()
    }

    override fun onDestroy() {
        super.onDestroy()
        subscription?.unsubscribe()
        subscription = null
    }

    override fun onResume() {
        super.onResume()
        binding.screenOrientation.value = Pref.Value.screen_orientation.name

        val callbackIconTheme = Pref3.icon_theme.value
        subscription?.add(Observable.fromCallable {
                Pref3.icon_theme.value.getIcon(activity) ?:
                if (Pref2.get().usesAdaptive())
                    NovaAdaptiveIconDrawable.create(resources, R.drawable.ic_launcher_default_adaptive, null)!!
                else
                    activity.getDrawable(R.mipmap.ic_launcher_default)
            }
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())
            .subscribe {
                if (callbackIconTheme == Pref3.icon_theme.value)
                    binding.iconStyle.widgetIcon = it
            })

        val gestureName = SettingsGestures.getPreferenceForAction(NovaAction.TOGGLE_STATUS_BAR)
        if (gestureName == null) {
            binding.notificationBarHint.visibility = View.GONE
        } else {
            binding.notificationBarHint.visibility = View.VISIBLE
            if (gestureName.iconId != 0)
                binding.notificationBarHint.setWidgetIconRes(gestureName.iconId)
            else
                binding.notificationBarHint.widgetIcon = null
            binding.notificationBarHint.title = getString(R.string.preference_gesture_configured_toggle, getString(gestureName.name))

            binding.notificationBarHint.setOnClickListener {
                val f = SettingsGestures()
                f.arguments = Bundle().apply {
                    putBundle("intentArgs", Bundle().apply {
                        putInt("targetPref", gestureName.prefId)
                    })
                }
                (activity).fragmentManager.beginTransaction()
                        .replace(R.id.fragment_container, f, "SETTINGS_FRAGMENT")
                        .setTransition(FragmentTransaction.TRANSIT_FRAGMENT_OPEN)
                        .addToBackStack(null)
                        .commit()
            }

        }

    }

    private fun updatePopupMenuSummary() {
        val sb = StringBuilder("%s ")
        val v = Pref3.popup_menu_item_visibility.value
        val list = ArrayList<String>()
        if (v.uninstall.desktop || v.uninstall.drawer)
            list.add(getString(R.string.delete_target_uninstall_label))
        if (v.playStore.desktop || v.playStore.drawer)
            list.add(getString(R.string.play_store))
        if (v.relaunch.desktop || v.relaunch.drawer)
            list.add(getString(R.string.popup_menu_kill_and_launch))
        if (list.isNotEmpty())
            sb.append(list.joinToString(", ", prefix = "(", postfix = ")"))
        binding.popupMenuStyle.summary = sb.toString()
    }

    private fun showDevAnimationSpeedDialog() {
        var devOptions = "Developer options"
        try {
            val res = activity.packageManager.getResourcesForApplication("com.android.settings")
            val resId = res.getIdentifier("development_settings_title", "string", "com.android.settings")
            if (resId != 0) {
                devOptions = res.getString(resId)
            }
        } catch (e: PackageManager.NameNotFoundException) {
        }

        AlertDialog.Builder(activity)
                .setMessage(activity.getString(R.string.animation_speed_developer_options))
                .setNegativeButton(R.string.cancel, null)
                .setPositiveButton(devOptions) { dialog, which ->
                    val intent = Intent(Settings.ACTION_APPLICATION_DEVELOPMENT_SETTINGS)
                    intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TASK)
                    startActivity(intent)
                }.show()
    }
}