package com.teslacoilsw.launcher.preferences.fragments

import android.Manifest
import android.content.pm.PackageManager
import android.os.Build
import android.os.Bundle
import android.text.format.DateFormat
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import androidx.core.content.ContextCompat
import androidx.core.view.isVisible
import com.teslacoilsw.kotlin.bind
import com.teslacoilsw.launcher.R
import com.teslacoilsw.launcher.databinding.SettingsNightmodeBinding
import com.teslacoilsw.launcher.daynight.DayNightManager
import com.teslacoilsw.launcher.preferences.NightMode
import com.teslacoilsw.launcher.preferences.Pref2
import com.teslacoilsw.launcher.preferences.Pref3
import com.wdullaer.materialdatetimepicker.time.TimePickerDialog
import java.util.*

private const val REQUEST_PERMISSION_LOCATION = 5913
class SettingsNightMode: NovaSettingsFragment() {
    private lateinit var binding: SettingsNightmodeBinding

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup, savedInstanceState: Bundle?): View {
        binding = inflater.bind(R.layout.settings_nightmode, container, false)
        binding.nightModeSchedule.setOnLongClickListener {
            if (Pref3.night_mode.value == NightMode.DEFAULT)
                Pref3.night_mode.value = NightMode.AUTO_DEBUG
            else
                Pref3.night_mode.value = NightMode.DEFAULT
            true
        }
        binding.nightModeSchedule.onUserChanged = { old, new ->
            if (NightMode.CUSTOM.name == new) {
                val flatSunset = Pref2.get().night_mode_custom_sunset.get()!!
                val flatSunrise = Pref2.get().night_mode_custom_sunrise.get()!!

                val tpd = TimePickerDialog.newInstance(
                        { view, sunsetHour, sunsetMinute, second ->
                            val tpd1 = TimePickerDialog.newInstance(
                                    { view1, sunriseHour, sunriseMinute, second1 ->
                                        Pref2.get().night_mode_custom_sunset.set(sunsetHour * 60 + sunsetMinute)
                                        Pref2.get().night_mode_custom_sunrise.set(sunriseHour * 60 + sunriseMinute)
                                        Pref3.night_mode.value = NightMode.CUSTOM
                                    },
                                    flatSunrise / 60,
                                    flatSunrise % 100,
                                    DateFormat.is24HourFormat(activity))
                            if (sunsetMinute > 0) {
                                tpd1.setMaxTime(sunsetHour, sunsetMinute - 1, 0)
                            } else {
                                if (sunsetHour == 0)
                                    tpd1.setMaxTime(23, 59, 0)
                                else
                                    tpd1.setMaxTime(sunsetHour - 1, 59, 0)
                            }
                            tpd1.title = activity.getString(R.string.nightmode_sunrise_time)
                            tpd1.isThemeDark = false
                            tpd1.show(fragmentManager, "timepickerdialog")

                        },
                        flatSunset / 60,
                        flatSunset % 100,
                        DateFormat.is24HourFormat(activity))
                tpd.title = activity.getString(R.string.nightmode_sunset_time)
                tpd.isThemeDark = true
                tpd.show(fragmentManager, "timepickerdialog")
            }

            DayNightManager.get().invalidate()
            updateNightModeSummary()
            requireRestart()
        }

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            binding.nightModeLocation.setEnableCondition(Pref3.night_mode.asObservable.map { it == NightMode.AUTO })
            binding.nightModeLocation.isChecked = Pref3.night_mode_location.value && hasLocationPermission()
            binding.nightModeLocation.onUserChanged = { _, useLocation ->
                if (useLocation && !hasLocationPermission()) {
                    requestPermissions(arrayOf(Manifest.permission.ACCESS_COARSE_LOCATION, Manifest.permission.ACCESS_FINE_LOCATION), REQUEST_PERMISSION_LOCATION)
                    binding.nightModeLocation.isChecked = false
                }
                Pref3.night_mode_location.value = binding.nightModeLocation.isChecked
                updateNightModeSummary()
            }
        } else {
            binding.nightModeLocation.isVisible = false
        }


        return binding.root
    }


    private fun hasLocationPermission(): Boolean {
        return ContextCompat.checkSelfPermission(activity, android.Manifest.permission.ACCESS_COARSE_LOCATION) == PackageManager.PERMISSION_GRANTED
    }

    override fun onResume() {
        super.onResume()
        updateNightModeSummary()

    }

    private fun updateNightModeSummary() {
        if (Pref3.night_mode.value == NightMode.AUTO) {
            val daynight = DayNightManager.get()
            val summary = daynight.getSunsetSummary(activity)
            if (binding.nightModeLocation.isEnabled && !binding.nightModeLocation.isChecked) {
                binding.nightModeSchedule.summary = "[TZ: " + TimeZone.getDefault().id + "]\n" + summary
            } else {
                binding.nightModeSchedule.summary = "$summary"
            }
        } else {
            binding.nightModeSchedule.summary = null
        }
    }


    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<String>, grantResults: IntArray) {
        if (REQUEST_PERMISSION_LOCATION == requestCode) {
            var granted = false
            for (g in grantResults) {
                if (g == PackageManager.PERMISSION_GRANTED) {
                    granted = true
                    break
                }
            }
            if (granted) {
                Pref3.night_mode_location.value = true
                binding.nightModeLocation.isChecked = true
            } else {
                Toast.makeText(activity, R.string.permission_denied, Toast.LENGTH_SHORT).show()
            }
        } else {
            super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        }
    }

}