package com.teslacoilsw.launcher.preferences.widget

import android.content.Context
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.graphics.Canvas
import android.graphics.Color
import android.preference.Preference
import android.util.AttributeSet
import android.view.View
import android.widget.ImageView
import com.android.launcher3.BubbleTextView
import com.android.launcher3.FastBitmapDrawable
import com.android.launcher3.FolderIcon
import com.teslacoilsw.kotlin.lazy1
import com.teslacoilsw.launcher.FolderIconHelper
import com.teslacoilsw.launcher.R
import com.teslacoilsw.launcher.util.DpUtil
import com.teslacoilsw.matcompat.TintCompat

class FolderIconPreference(context: Context, attrs: AttributeSet?) : Preference(context, attrs) {
    private var view: View? = null
    private lateinit var imageView: ImageView
    private val folderIconHelper by lazy1 { FolderIconHelper(context) }

    private val folderIconSize by lazy1 { DpUtil.getPixelSize(48) }
    private val appIconSize by lazy1 { (folderIconSize/ FolderIcon.FOLDER_ICON_SCALE).toInt() }

    private val previewBitmap by lazy1 { Bitmap.createBitmap(folderIconSize, folderIconSize, Bitmap.Config.ARGB_8888) }
    private val previewCanvas = Canvas()
    private val previewItems by lazy1 { listOf(
            BubbleTextView(context).apply { applyCompoundDrawables(FastBitmapDrawable(BitmapFactory.decodeResource(resources, R.drawable.ic_lptheme_settings)).apply { setBounds(0, 0, appIconSize, appIconSize) }) },
            BubbleTextView(context).apply { applyCompoundDrawables(FastBitmapDrawable(BitmapFactory.decodeResource(resources, R.drawable.ic_lptheme_email)).apply { setBounds(0, 0, appIconSize, appIconSize) }) },
            BubbleTextView(context).apply { applyCompoundDrawables(FastBitmapDrawable(BitmapFactory.decodeResource(resources, R.drawable.ic_lptheme_clock)).apply { setBounds(0, 0, appIconSize, appIconSize) }) },
            BubbleTextView(context).apply { applyCompoundDrawables(FastBitmapDrawable(BitmapFactory.decodeResource(resources, R.drawable.ic_lptheme_sms)).apply { setBounds(0, 0, appIconSize, appIconSize) }) }
    ) }

    init {
        widgetLayoutResource = R.layout.preference_widget_imageview
    }

    override fun onBindView(view: View) {
        this.view = view
        super.onBindView(view)
        imageView = view.findViewById(R.id.preference_widget_imageview)
//        imageView.setBackgroundColor(ContextCompat.getColor(context, R.color.searchbarbuilder_bar_style_background))
        updatePreview()
    }

    fun updatePreview() {
        if (!::imageView.isInitialized)
            return

        folderIconHelper.setConfigFromPreferences(context)
        previewBitmap.eraseColor(Color.TRANSPARENT)
        previewCanvas.setBitmap(previewBitmap)

        folderIconHelper.computePreviewDrawingParams(folderIconSize, appIconSize, folderIconSize)
        val windowColor = TintCompat.getThemeAttrColor(context, android.R.attr.windowBackground)
        folderIconHelper.drawFolderIcon(previewCanvas, previewItems, null, true, false, null, windowColor, false)
        previewCanvas.setBitmap(null)
        imageView.setImageDrawable(FastBitmapDrawable(previewBitmap).apply { setBounds(0, 0, folderIconSize, folderIconSize) })
    }
}