package com.teslacoilsw.launcher.util

import io.reactivex.disposables.Disposable
import io.reactivex.subjects.BehaviorSubject

/**
 * Wrapper class for a value that's changes can be observed
 */
class ObservableValue<T>(value: T) {
    var value: T = value
        set(newValue) {
            field = newValue
            if (observable.hasObservers()) {
                observable.onNext(newValue)
            }
        }

    val observable: BehaviorSubject<T> = BehaviorSubject.createDefault(value)

    fun subscribe(onNext: (T) -> Unit): Disposable = observable.subscribe(onNext)
    fun subscribe(onNext: (T) -> Unit, onError: (Throwable) -> Unit): Disposable = observable.subscribe(onNext, onError)
}