package com.teslacoilsw.launcher.util.StatusBar

import android.Manifest
import android.content.ContentResolver
import android.content.pm.PackageManager
import android.os.SystemClock
import android.provider.Settings
import androidx.core.content.ContextCompat
import com.android.launcher3.LauncherAppState
import com.teslacoilsw.launcher.NovaLauncher
import com.teslacoilsw.launcher.preferences.Pref3
import com.teslacoilsw.shared.util.rewthelper.RewtHelper
import java.io.IOException

enum class HideClock {
    DISABLED {
        override fun update(launcher: NovaLauncher) {
            if (hasSystemPriv()) {
                launcher.mStatusBar.privAppDisable(StatusBarManager.STATUS_BAR_DISABLE_NONE)
            } else {
                launcher.bgHandler.removeAndPostOne(hideClockRunnable, showClockRunnable)
            }
        }
    },
    ALL {
        override fun update(launcher: NovaLauncher) {
            if (launcher.isActivityFocusedResumed) {
                if (hasSystemPriv()) {
                    launcher.mStatusBar.privAppDisable(StatusBarManager.STATUS_BAR_DISABLE_CLOCK)
                } else {
                    launcher.bgHandler.removeAndPostOne(showClockRunnable, hideClockRunnable)
                }
            } else {
                if (hasSystemPriv()) {
                    launcher.mStatusBar.privAppDisable(StatusBarManager.STATUS_BAR_DISABLE_NONE)
                } else {
                    launcher.bgHandler.removeAndPostOne(hideClockRunnable, showClockRunnable)
                }
            }
        }
    },
    DESKTOP {
        override fun update(launcher: NovaLauncher) {
            if (launcher.isActivityFocusedResumed && !launcher.isAllAppsVisible) {
                if (hasSystemPriv()) {
                    launcher.mStatusBar.privAppDisable(StatusBarManager.STATUS_BAR_DISABLE_CLOCK)
                } else {
                    launcher.bgHandler.removeAndPostOne(showClockRunnable, hideClockRunnable)
                }
            } else {
                if (hasSystemPriv()) {
                    launcher.mStatusBar.privAppDisable(StatusBarManager.STATUS_BAR_DISABLE_NONE)
                } else {
                    launcher.bgHandler.removeAndPostOne(hideClockRunnable, showClockRunnable)
                }
            }
        }
    },
    HOMEPAGE {
        override fun update(launcher: NovaLauncher) {
            if (launcher.isActivityFocusedResumed && !launcher.isAllAppsVisible && launcher.isAtDefaultScreen) {
                if (hasSystemPriv()) {
                    launcher.mStatusBar.privAppDisable(StatusBarManager.STATUS_BAR_DISABLE_CLOCK)
                } else {
                    launcher.bgHandler.removeAndPostOne(showClockRunnable, hideClockRunnable)
                }
            } else {
                if (hasSystemPriv()) {
                    launcher.mStatusBar.privAppDisable(StatusBarManager.STATUS_BAR_DISABLE_NONE)
                } else {
                    launcher.bgHandler.removeAndPostOne(hideClockRunnable, showClockRunnable)
                }
            }
        }
    };

    abstract fun update(launcher: NovaLauncher)

    companion object {

        private var sClockHidden = false
        private var sHasSystemPriv: Boolean? = null
        private var sHasAdbPriv: Boolean? = null

        @JvmStatic
        fun hasSystemPriv(): Boolean {
            if (sHasSystemPriv == null) {
                val context = LauncherAppState.getInstance().context
                sHasSystemPriv = LauncherAppState.isSystemApp(context) && ContextCompat.checkSelfPermission(context, Manifest.permission.STATUS_BAR) == PackageManager.PERMISSION_GRANTED
            }

            return sHasSystemPriv!!
        }

        fun hasAdbPriv(): Boolean {
            if (sHasAdbPriv == null) {
                val context = LauncherAppState.getInstance().context
                sHasAdbPriv = ContextCompat.checkSelfPermission(context, Manifest.permission.WRITE_SECURE_SETTINGS) == PackageManager.PERMISSION_GRANTED
            }
            return sHasAdbPriv!!
        }

        private fun updateBlacklist(cr: ContentResolver, showClock: Boolean) {
            var currentBlacklist = Settings.Secure.getString(cr, "icon_blacklist") ?: ""
            val items = currentBlacklist.split(',').dropLastWhile { it.isEmpty() }.toMutableSet()
            if (showClock) {
                items.remove("clock")
            } else {
                items.add("clock")
            }

            try {
                Settings.Secure.putString(cr, "icon_blacklist", items.joinToString(","))
            } catch (ex: Exception) {
                ex.printStackTrace()
            }

        }

        private val showClockRunnable = Runnable {
            if (sClockHidden) {
                if (hasAdbPriv()) {
                    updateBlacklist(LauncherAppState.getInstance().context.contentResolver, true)
                } else {
                    val rewt = RewtHelper.getInstance(LauncherAppState.getInstance().context)
                    if (rewt.isReady) {
                        rewt.disableStatusBar(RewtHelper.StatusBar.DISABLE_NONE)
                    }
                }
                sClockHidden = false
            }
        }

        private val hideClockRunnable = Runnable {
            if (!sClockHidden) {
                if (hasAdbPriv()) {
                    updateBlacklist(LauncherAppState.getInstance().context.contentResolver, false)
                    sClockHidden = true
                } else {
                    val rewt = RewtHelper.getInstance(LauncherAppState.getInstance().context)
                    if (!rewt.isReady) {
                        sClockHidden = false
                        try {
                            rewt.setupIfNeeded()
                        } catch (e: IOException) {
                            Pref3.notification_bar_hide_clock.value = DISABLED
                            e.printStackTrace()
                            return@Runnable
                        }

                        /* Root might not be ready at boot, but after 5 minutes it should be */
                        if (!rewt.isReady && SystemClock.uptimeMillis() > 5 * 60 * 1000) {
                            Pref3.notification_bar_hide_clock.value = DISABLED
                            return@Runnable
                        }
                    }
                    rewt.disableStatusBar(RewtHelper.StatusBar.STATUS_BAR_DISABLE_CLOCK)
                    sClockHidden = true
                }
            }
        }
    }
}