package com.teslacoilsw.launcher.widget

import android.content.Context
import android.content.res.ColorStateList
import android.graphics.Canvas
import android.graphics.drawable.GradientDrawable
import android.os.Build
import android.util.AttributeSet
import android.widget.SeekBar
import com.teslacoilsw.kotlin.dp
import com.teslacoilsw.kotlin.getThemeAttrColor
import com.teslacoilsw.launcher.R
import com.teslacoilsw.launcher.util.palette.ColorUtils

private val Int.csl: ColorStateList
    get() = ColorStateList.valueOf(this)
class DiscreteSeekBar @JvmOverloads constructor(
        context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0
) : SeekBar(context, attrs, defStyleAttr, R.style.SeekBar) {
    private val tickMarkDrawable = GradientDrawable().apply {
        shape = GradientDrawable.OVAL
    }
    private val tickMarkSize = context.dp(3)

    override fun onDraw(canvas: Canvas) {
        super.onDraw(canvas)
        drawTickMarks(canvas)
    }

    /**
     * Draw the tick marks.
     */
    private fun drawTickMarks(canvas: Canvas) {
        val themeColor = context.getThemeAttrColor(R.attr.colorControlActivated)
        val count = max
        if (count > 1) {
            val w = tickMarkSize
            val h = tickMarkSize
            val halfW = if (w >= 0) w / 2 else 1
            val halfH = if (h >= 0) h / 2 else 1
            tickMarkDrawable.setBounds(-halfW, -halfH, halfW, halfH)

            val spacing = (width - paddingLeft
                    - paddingRight) / count.toFloat()
            val saveCount = canvas.save()
            canvas.translate(paddingLeft.toFloat(), height / 2f)
            val progressCount = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O)
                progress - min
            else
                progress

            for (i in 0..count) {
                if (i < progressCount) {
                    tickMarkDrawable.color = ColorUtils.compositeColors(0x88FFFFFF.toInt(), themeColor).csl
                } else if (i == progressCount) {
                    tickMarkDrawable.color = themeColor.csl
                } else {
                    tickMarkDrawable.color = ColorUtils.compositeColors(0x45FFFFFF.toInt(), themeColor).csl
                }
                tickMarkDrawable.draw(canvas)
                canvas.translate(spacing, 0f)
            }
            canvas.restoreToCount(saveCount)
        }
    }
}