package com.teslacoilsw.launcher.widget

import android.annotation.SuppressLint
import android.graphics.Bitmap
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.Paint
import android.graphics.PorterDuff
import android.graphics.PorterDuffXfermode
import android.os.Build
import androidx.core.graphics.alpha
import com.teslacoilsw.kotlin.lazy1
import com.teslacoilsw.kotlin.resetAfter
import com.teslacoilsw.kotlin.withAlpha
import com.teslacoilsw.launcher.util.OEMUtils

private val IS_P = OEMUtils.isAtLeast(Build.VERSION_CODES.O_MR1, 2)
class FadingEdgeScrimHelper {
    private val paint by lazy1 {
        if (IS_P) {
            Paint().apply {
                alpha = 128
            }
        } else {
            Paint().apply {
                color = 0x80FFFFFF.toInt(); xfermode = PorterDuffXfermode(PorterDuff.Mode.DST_OUT) }
        }
    }
    private var topBmp: Bitmap? = null
    private var bottomBmp: Bitmap? = null
    private val tempCanvas = if (IS_P) Canvas() else null

    private var topScrimHeight = 0
    private var bottomScrimHeight = 0

    fun setScrimHeight(top: Int, bottom: Int) {
        topScrimHeight = top
        bottomScrimHeight = bottom
    }

    @SuppressLint("NewApi")
    @Suppress("DEPRECATION")
    fun dispatchDraw(canvas: Canvas, left: Int, top: Int, right: Int, bottom: Int, bgColor: Int, body: (Canvas) -> Unit) {
        canvas.resetAfter {
            var save = 0
            if (bgColor.alpha != 255) {
                if (OEMUtils.isAtLeast(Build.VERSION_CODES.O_MR1, 2)) {
                    save = canvas.save()
                    if (topScrimHeight > 0)
                        canvas.clipOutRect(left.toFloat(), top.toFloat(), right.toFloat(), (top + topScrimHeight).toFloat())

                    if (bottomScrimHeight > 0)
                        canvas.clipOutRect(left.toFloat(), (bottom - bottomScrimHeight).toFloat(), right.toFloat(), bottom.toFloat())
                } else {
                    if (topScrimHeight > 0)
                        canvas.saveLayer(left.toFloat(), top.toFloat(), right.toFloat(), (top + topScrimHeight).toFloat(), null, 0)
                    if (bottomScrimHeight > 0)
                        canvas.saveLayer(left.toFloat(), (bottom - bottomScrimHeight).toFloat(), right.toFloat(), bottom.toFloat(), null, 0)
                }
            }

            body(canvas)

            if (save != 0)
                canvas.restoreToCount(save)

            if (bgColor.alpha != 255) {
                if (OEMUtils.isAtLeast(Build.VERSION_CODES.O_MR1, 2)) {
                    val width = right - left
                    var tbmp = topBmp
                    var bbmp = bottomBmp
                    if (topScrimHeight > 0) {
                        if (tbmp == null || tbmp.height != topScrimHeight || tbmp.width != width) {
                            tbmp = Bitmap.createBitmap(width, topScrimHeight, Bitmap.Config.ARGB_8888)
                            topBmp = tbmp
                        }
                        tbmp!!.eraseColor(Color.TRANSPARENT)
                        tempCanvas!!.setBitmap(tbmp)
                        body(tempCanvas)
                        tempCanvas.setBitmap(null)
                        canvas.drawBitmap(tbmp, left.toFloat(), top.toFloat(), paint)
                    }

                    if (bottomScrimHeight > 0) {
                        if (bbmp == null || bbmp.height != bottomScrimHeight || bbmp.width != width) {
                            bbmp = Bitmap.createBitmap(width, bottomScrimHeight, Bitmap.Config.ARGB_8888)
                            bottomBmp = bbmp
                        }
                        bbmp!!.eraseColor(Color.TRANSPARENT)
                        tempCanvas!!.setBitmap(bbmp)
                        tempCanvas.translate(0f, -(bottom - bottomScrimHeight).toFloat())
                        body(tempCanvas)
                        tempCanvas.setBitmap(null)
                        canvas.drawBitmap(bbmp, left.toFloat(), (bottom - bottomScrimHeight).toFloat(), paint)
                    }
                } else {
                    if (topScrimHeight > 0)
                        canvas.drawRect(left.toFloat(), top.toFloat(), right.toFloat(), (top + topScrimHeight).toFloat(), paint)

                    if (bottomScrimHeight > 0)
                        canvas.drawRect(left.toFloat(), (bottom - bottomScrimHeight).toFloat(), right.toFloat(), bottom.toFloat(), paint)
                }
            } else {
                if (topScrimHeight > 0) {
                    val save = canvas.save()
                    canvas.clipRect(left.toFloat(), top.toFloat(), right.toFloat(), top.toFloat() + topScrimHeight)
                    canvas.drawColor(bgColor.withAlpha(0x80))
                    canvas.restoreToCount(save)
                }
                if (bottomScrimHeight > 0) {
                    val save = canvas.save()
                    canvas.clipRect(left.toFloat(), (bottom - bottomScrimHeight).toFloat(), right.toFloat(), bottom.toFloat())
                    canvas.drawColor(bgColor.withAlpha(0x80))
                    canvas.restoreToCount(save)
                }
            }
        }
    }

}